/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.applet;

import java.applet.Applet;
import java.awt.BorderLayout;
import pl.chyla.wyjec.Config;
import pl.chyla.wyjec.apphelpers.SimpleParser;
import pl.chyla.wyjec.core.Value;
import pl.chyla.wyjec.evaluator.EvaluationException;
import pl.chyla.wyjec.evaluator.Evaluator;
import pl.chyla.wyjec.parser.ExpressionTree;
import pl.chyla.wyjec.parser.InvalidInputException;


/**
 * Simple applet for evaluating Wyjec expressions.
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
public final class AppletEvaluator extends Applet {

    /**
     * The name of this applet's parameter specifying Wyjec expression to display in the GUI
     * component after initialization (optional).
     */
    private static final String EXPRESSION_PARAM_NAME = "expression";

    /**
     * Wyjec parser used by this applet.
     */
    private final SimpleParser parser;

    /**
     * Wyjec evaluator used by this applet.
     */
    private final Evaluator evaluator;

    /**
     * Main GUI component of this applet.
     */
    private EvaluatorGUI gui;

    /**
     * Constructs <code>AppletEvaluator</code> instance, without initializing its GUI component.
     */
    public AppletEvaluator() {
        setLayout(new BorderLayout());
        parser = new SimpleParser();
        evaluator = new Evaluator(Config.getDefaultConfig());
    }

    /**
     * Called by this applet's "container". When called for the first time, reads Wyjec
     * expression given as this applet's parameter and uses it to instantiate the GUI component.
     */
    public void init() {
        if (gui != null) {
            return;
        }
        String defaultExpression = getParameter(EXPRESSION_PARAM_NAME);
        if (defaultExpression != null) {
            gui = new EvaluatorGUI(defaultExpression);
            evaluateExpression(defaultExpression);
        } else {
            gui = new EvaluatorGUI("");
        }
        add(gui, BorderLayout.CENTER);
        gui.addExpressionEditListener(new IExpressionEditListener() {
            public void changed(String expression) {
                gui.clearResult();
            }
            public void accepted(String expression) {
                evaluateExpression(expression);
            }
        });
    }

    /**
     * Evaluates Wyjec expression contained in given string and passes the result or error
     * information to the GUI component.
     *
     * @param expression expression to evaluate
     */
    private void evaluateExpression(String expression) {
        ExpressionTree exprTree;
        try {
            exprTree = parser.parse(expression);
        } catch (InvalidInputException e) {
            gui.showResultError("Error parsing the expression.\n" + e.getMessage());
            exprTree = null;
        }
        if (exprTree != null) {
            try {
                Value result = evaluator.evaluate(exprTree);
                gui.showResultValue(result);
            } catch (EvaluationException e) {
                StringBuffer errorBuf = new StringBuffer();
                errorBuf.append("Evaluation Error!\n");
                for (Throwable cause = e.getCause(); cause != null; cause = cause.getCause()) {
                    String msg = cause.getMessage();
                    if (msg != null) {
                        errorBuf.append(cause.getClass() + ": " + msg + "\n");
                    } else {
                        errorBuf.append(cause.getClass() + "\n");
                    }
                }
                gui.showResultError(errorBuf.toString());
            }
        }
    }

}
