/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.applet;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import pl.chyla.wyjec.core.Value;


/**
 * Swing GUI component responsible for interaction between Wyjec evaluator and the user.
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
final class EvaluatorGUI extends JComponent {

    /**
     * Message do display on result label when there's no result.
     */
    private static final String NO_RESULT_LABEL_INFO =
        "<html>No result (press <i>Enter</i>)</html>";

    /**
     * Field for editing an expression.
     */
    private final JTextField exprField;

    /**
     * Label associated with area containing the result of evaluation.
     */
    private final JLabel resultLabel;

    /**
     * Area containing the result of expression evaluation.
     */
    private final JTextPane resultPane;

    /**
     * A list of listeners to notify about expression editing events.
     */
    private final List expressionEditListeners;

    /**
     * Constructs <code>EvaluatorGUI</code> instance with given default expression string to
     * use.
     *
     * @param defaultExpression default value of expression to display in editing area
     */
    EvaluatorGUI(String defaultExpression) {
        setLayout(new GridBagLayout());
        setBorder(BorderFactory.createEtchedBorder());
        GridBagConstraints c = new GridBagConstraints();
        c.insets = new Insets(2, 2, 2, 2);
        c.weightx = 1.0;
        c.gridx = 0;
        c.gridy = GridBagConstraints.RELATIVE;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.weighty = 0.0;
        JLabel exprLabel = new JLabel("Expression:");
        add(exprLabel, c);
        c.weighty = 0.0;
        c.fill = GridBagConstraints.HORIZONTAL;
        exprField = new JTextField(defaultExpression);
        add(exprField, c);
        c.weighty = 0.0;
        c.fill = GridBagConstraints.HORIZONTAL;
        resultLabel = new JLabel(NO_RESULT_LABEL_INFO);
        add(resultLabel, c);
        c.weighty = 1.0;
        c.fill = GridBagConstraints.BOTH;
        resultPane = new JTextPane();
        resultPane.setEditable(false);
        add(resultPane, c);
        expressionEditListeners = new LinkedList();
        exprField.getDocument().addDocumentListener(new DocumentListener() {
            public void changedUpdate(DocumentEvent e) {
                fireExpressionEditChanged(exprField.getText());
            }
            public void insertUpdate(DocumentEvent e) {
                fireExpressionEditChanged(exprField.getText());
            }
            public void removeUpdate(DocumentEvent e) {
                fireExpressionEditChanged(exprField.getText());
            }
        });
        exprField.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                fireExpressionEditAccepted(exprField.getText());
            }
        });
    }

    /**
     * Registers a listener to notify about expression editing events.
     *
     * @param listener listener to register
     */
    public void addExpressionEditListener(IExpressionEditListener listener) {
        expressionEditListeners.add(listener);
    }

    /**
     * Notifies all registered listeners that an expression inside editing area has been
     * changed by the user.
     *
     * @param expression current value of expression string from editing area
     */
    private void fireExpressionEditChanged(String expression) {
        for (Iterator iter = expressionEditListeners.iterator(); iter.hasNext();) {
            IExpressionEditListener listener = (IExpressionEditListener) iter.next();
            listener.changed(exprField.getText());
        }
    }

    /**
     * Notifies all registered listeners that an expression inside editing area has been
     * accepted by the user.
     *
     * @param expression current value of expression string from editing area
     */
    private void fireExpressionEditAccepted(String expression) {
        for (Iterator iter = expressionEditListeners.iterator(); iter.hasNext();) {
            IExpressionEditListener listener = (IExpressionEditListener) iter.next();
            listener.accepted(exprField.getText());
        }
    }

    /**
     * Displays given value as the result of expression evaluation inside the result area.
     *
     * @param result result of evaluation
     */
    public void showResultValue(Value result) {
        String typeStr = result.getType().getName();
        resultLabel.setText("<html>Result (type: <i>" + typeStr + "</i>):</html>");
        resultPane.setText(result.toString());
    }

    /**
     * Displays information about specified error inside the result area.
     *
     * @param message error message to display
     */
    public void showResultError(String message) {
        resultLabel.setText("Error:");
        resultPane.setText(message);
    }

    /**
     * Clears content of the result area.
     */
    public void clearResult() {
        resultLabel.setText(NO_RESULT_LABEL_INFO);
        resultPane.setText("");
    }

}
