/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.cli;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import pl.chyla.wyjec.core.Value;
import pl.chyla.wyjec.funcinfo.FunctionInformation;
import pl.chyla.wyjec.funcinfo.ParameterInformation;
import pl.chyla.wyjec.simulation.IFunctionHandler;


/**
 * Implementation of handler for simulated functions that obtains results of simulated functions
 * evaluations from the user, using command line interface.
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
final class CLIFunctionHandler implements IFunctionHandler {

    /**
     * Default empty constructor.
     */
    CLIFunctionHandler() { }

    /**
     * Simulates evaluation of a specified function by obtaining its value from the user using
     * command line interface. The method displays the information about the function and its
     * actual parameters and then asks for the result of simulated evaluation.
     *
     * @param funcInfo information about the evaluated function
     * @param result object to store evaluation result to
     * @param params actual parameters of the function
     */
    public void evaluateFunction(FunctionInformation funcInfo, Value result, Value[] params) {
        System.out.println("Simulated function: " + funcInfo.getDeclarationDescription());
        System.out.println("Actual parameters:  " + getActParamsDescription(funcInfo, params));
        System.out.println(
            "Enter function's result (" + funcInfo.getResultType().getName() + "):");
        String line;
        try {
            line = readLine(System.in);
        } catch (IOException e) {
            line = "";
        }
        ValueParser.parseValue(result, line, funcInfo.getResultType());
    }

    /**
     * Returns description of a sequence of actual parameters for specified function, using both
     * parameters' names from the function information and their specified values. The
     * description will contain a sequence of "name = value" pairs with values of optional
     * parameters grouped together.
     *
     * @param funcInfo information about the function
     * @param actParams values of the actual parameters
     * @return string containing description of the parameters
     */
    private String getActParamsDescription(FunctionInformation funcInfo, Value[] actParams) {
        StringBuffer desc = new StringBuffer();
        ParameterInformation[] reqParamsInfos = funcInfo.getRequiredParameters();
        for (int i = 0; i < reqParamsInfos.length; i++) {
            if (i > 0) {
                desc.append(", ");
            }
            desc.append(reqParamsInfos[i].getName());
            desc.append(" = ");
            desc.append(actParams[i].toString());
        }
        ParameterInformation optParamsInfo = funcInfo.getOptionalParameters();
        if (optParamsInfo != null) {
            if (reqParamsInfos.length != 0) {
                desc.append(", ");
            }
            desc.append(optParamsInfo.getName());
            desc.append(" = [");
            for (int i = reqParamsInfos.length; i < actParams.length; i++) {
                if (i > reqParamsInfos.length) {
                    desc.append(", ");
                }
                desc.append(actParams[i]);
            }
            desc.append("]");
        }
        return desc.toString();
    }

    /**
     * Reads single line from given input stream, stripping end-of-line character if necessary.
     *
     * @param input input stream to read a line from
     * @return string containing single line read from the stream
     * @throws IOException in case of error reading from the stream
     */
    private String readLine(InputStream input) throws IOException {
        ByteArrayOutputStream line = new ByteArrayOutputStream();
        do {
            int b = input.read();
            if (b == -1 || b == '\n') {
                break;
            }
            line.write(b);
        } while (true);
        return new String(line.toByteArray());
    }

}
