/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.cli;

import pl.chyla.wyjec.utils.cli.CommandLine;
import pl.chyla.wyjec.utils.cli.CommandLineParser;
import pl.chyla.wyjec.utils.cli.CommandLineParser.CommandLineSyntaxException;


/**
 * Class responsible for parsing command line options specific for the evaluator and providing
 * them via getters.
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
final class CLIOptionsProvider {

    /**
     * Name of the option used to disable standard functions.
     */
    private static final String NO_STDFUNCS_OPTION = "-nostdfuncs";

    /**
     * Name of the option used to disable standard constants.
     */
    private static final String NO_STDCONSTS_OPTION = "-nostdconsts";

    /**
     * Name of the option used to specify a name of functions factory class.
     */
    private static final String FUNCS_CLASS_OPTION = "-fclass";

    /**
     * Name of the option used to specify a name of XML file containing functions descriptions
     * used to build simulated functions.
     */
    private static final String FUNCS_DOC_OPTION = "-fdoc";

    /**
     * Name of the option used to specify a name of XML file containing functions descriptions
     * used to test their example expressions.
     */
    private static final String FUNCS_EXAMPLES_DOC_OPTION = "-fexdoc";

    /**
     * Flag corresponding to {@value #NO_STDFUNCS_OPTION} option (set to <code>true</code> if
     * the option was used).
     */
    private final boolean optNoStdFuncs;

    /**
     * Flag corresponding to {@value #NO_STDCONSTS_OPTION} option (set to <code>true</code> if
     * the option was used).
     */
    private final boolean optNoStdConsts;

    /**
     * An array of all names of classes specified using {@value #FUNCS_CLASS_OPTION} options.
     */
    private final String[] optFuncsClasses;

    /**
     * An array of all names of XML files specified using {@value #FUNCS_DOCS_OPTION} options.
     */
    private final String[] optFuncsDocs;

    /**
     * An array of all names of XML files specified using {@value #FUNCS_EXAMPLES_DOC_OPTION}
     * options.
     */
    private final String[] optFuncsExamplesDocs;

    /**
     * An array of all real arguments of the program, not recognized as options or options'
     * arguments.
     */
    private final String[] arguments;

    /**
     * Parses given command line arguments, separates known command line options and their
     * arguments from the real arguments of the program and stores this data in a structured
     * way.
     *
     * @param inputArgs an array of arguments passed from the execution environment
     * @throws CommandLineSyntaxException in case of error parsing the command line
     */
    CLIOptionsProvider(String[] inputArgs) throws CommandLineSyntaxException {
        CommandLineParser parser = new CommandLineParser(
            new String[] {NO_STDFUNCS_OPTION, NO_STDCONSTS_OPTION},
            new String[] {FUNCS_CLASS_OPTION, FUNCS_DOC_OPTION, FUNCS_EXAMPLES_DOC_OPTION});
        CommandLine cLine = parser.parse(inputArgs);
        optNoStdFuncs = cLine.isOptionSet(NO_STDFUNCS_OPTION);
        optNoStdConsts = cLine.isOptionSet(NO_STDCONSTS_OPTION);
        optFuncsClasses = cLine.getOptionArguments(FUNCS_CLASS_OPTION);
        optFuncsDocs = cLine.getOptionArguments(FUNCS_DOC_OPTION);
        optFuncsExamplesDocs = cLine.getOptionArguments(FUNCS_EXAMPLES_DOC_OPTION);
        arguments = cLine.getArguments();
    }

    /**
     * Returns information about the use of {@value #NO_STDFUNCS_OPTION} option in the command
     * line.
     *
     * @return <code>true</code> if the option was used, <code>false</code> otherwise
     */
    public boolean getOptNoStdFuncs() {
        return optNoStdFuncs;
    }

    /**
     * Returns information about the use of {@value #NO_STDCONSTS_OPTION} option in the command
     * line.
     *
     * @return <code>true</code> if the option was used, <code>false</code> otherwise
     */
    public boolean getOptNoStdConsts() {
        return optNoStdConsts;
    }

    /**
     * Returns all names of classes specified using {@value #FUNCS_CLASS_OPTION} options in the
     * command line.
     *
     * @return
     *     an array of arguments of {@value #FUNCS_CLASS_OPTION} options or an empty array of
     *     no such option was used; the returned array should not be modified
     */
    public String[] getOptFuncsClasses() {
        return optFuncsClasses;
    }

    /**
     * Returns all names of XML files specified using {@value #FUNCS_DOCS_OPTION} options in the
     * command line.
     *
     * @return
     *     an array of arguments of {@value #FUNCS_DOCS_OPTION} options or an empty array of no
     *     such option was used; the returned array should not be modified
     */
    public String[] getOptFuncsDocs() {
        return optFuncsDocs;
    }

    /**
     * Returns all names of XML files specified using {@value #FUNCS_EXAMPLES_DOC_OPTION}
     * options in the command line.
     *
     * @return
     *     an array of arguments of {@value #FUNCS_EXAMPLES_DOC_OPTION} options or an empty
     *     array of no such option was used; the returned array should not be modified
     */
    public String[] getOptFuncsExamplesDocs() {
        return optFuncsExamplesDocs;
    }

    /**
     * Returns the real arguments of the program, left after filtering out all options and their
     * arguments.
     *
     * @return the program's arguments; the returned array should not be modified
     */
    public String[] getArguments() {
        return arguments;
    }

}
