/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.cli;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import pl.chyla.wyjec.Debugging;
import pl.chyla.wyjec.core.Type;
import pl.chyla.wyjec.core.Value;
import pl.chyla.wyjec.utils.RawString;


/**
 * Parser for values of all Wyjec types. It translates values stored as strings to real
 * {@link Value} instances.
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
final class ValueParser {

    /**
     * Format of {@link Type#DATE} values recognized by the parser.
     */
    private static final DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd,HH:mm:ss");

    /**
     * Private empty constructor to disallow instantiating of the class.
     */
    private ValueParser() { }

    /**
     * Parses value of one of Wyjec types from given string and stores it in {@link Value}
     * instance.
     *
     * @param result
     *     object to store parsing result to; after successful parsing its type will be set to
     *     <code>valueType</code>
     * @param valueString string containing a value to parse
     * @param valueType type of value to assume
     * @throws InvalidValueFormatException TODO
     */
    public static void parseValue(Value result, String valueString, Type valueType) {
        switch (valueType.getEncodedType()) {
        case Type.BOOLEAN_ENCODED:
            result.setBoolean(parseBoolean(valueString));
            break;
        case Type.INTEGER_ENCODED:
            result.setInteger(parseInteger(valueString));
            break;
        case Type.STRING_ENCODED:
            result.setString(new RawString(valueString));
            break;
        case Type.FLOAT_ENCODED:
            result.setFloat(parseFloat(valueString));
            break;
        case Type.DATE_ENCODED:
            result.setDate(parseDate(valueString));
            break;
        default:
            Debugging.assumeNotReached();
        }
        if (Debugging.DEBUG_ENABLED) {
            Debugging.assume(
                result.getType() == valueType,
                "Parsing result must have the same type as required one");
        }
    }

    /**
     * Parses Wyjec boolean value written in string format.
     *
     * @param valueString string containing a value to parse
     * @return parsed boolean value
     */
    private static boolean parseBoolean(String valueString) {
        if (valueString.equals("true")) {
            return true;
        } else if (valueString.equals("false")) {
            return false;
        } else {
            throw new InvalidValueFormatException(valueString, Type.BOOLEAN);
        }
    }

    /**
     * Parses Wyjec integer value written in string format.
     *
     * @param valueString string containing a value to parse
     * @return parsed integer value
     */
    private static long parseInteger(String valueString) {
        try {
            return Long.parseLong(valueString);
        } catch (NumberFormatException e) {
            throw new InvalidValueFormatException(valueString, Type.INTEGER);
        }
    }

    /**
     * Parses Wyjec float value written in string format.
     *
     * @param valueString string containing a value to parse
     * @return parsed float value
     */
    private static double parseFloat(String valueString) {
        try {
            return Double.parseDouble(valueString);
        } catch (NumberFormatException e) {
            throw new InvalidValueFormatException(valueString, Type.FLOAT);
        }
    }

    /**
     * Parses Wyjec date value written in string format.
     *
     * @param valueString string containing a value to parse
     * @return parsed date value
     */
    private static long parseDate(String valueString) {
        try {
            return dateFormat.parse(valueString).getTime();
        } catch (ParseException e) {
            throw new InvalidValueFormatException(valueString, Type.DATE);
        }
    }


    /**
     * Exception indicating that a string value doesn't contain Wyjec value of specified type.
     *
     * @author Zbigniew Chyla (zbigniew@chyla.pl)
     */
    public static final class InvalidValueFormatException extends RuntimeException {

        /**
         * Constructs <code>InvalidValueFormatException</code> using given invalid string value
         * and the value's type used for parsing.
         *
         * @param valueString string that couldn't be parsed
         * @param valueType assumed type of value that should be stored in the string
         */
        private InvalidValueFormatException(String valueString, Type valueType) {
            super(
                "Invalid value for type " + valueType.getName() + ": \"" + valueString + "\"");
        }

    }

}
