/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.cli;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;
import pl.chyla.wyjec.Config;
import pl.chyla.wyjec.Debugging;
import pl.chyla.wyjec.core.Function;
import pl.chyla.wyjec.core.IConstantsParser;
import pl.chyla.wyjec.core.Value;
import pl.chyla.wyjec.evaluator.EvaluationException;
import pl.chyla.wyjec.evaluator.Evaluator;
import pl.chyla.wyjec.funcinfo.ExpressionExample;
import pl.chyla.wyjec.funcinfo.FunctionInformation;
import pl.chyla.wyjec.funcinfo.FunctionsDocument;
import pl.chyla.wyjec.funcinfoxml.FunctionsDocumentParser;
import pl.chyla.wyjec.funcinfoxml.FunctionsDocumentProcessingException;
import pl.chyla.wyjec.parser.ExpressionTree;
import pl.chyla.wyjec.parser.InvalidInputException;
import pl.chyla.wyjec.parser.Parser;
import pl.chyla.wyjec.simulation.SimulatedFunctionsFactory;
import pl.chyla.wyjec.stdconsts.EmptyConstantsParser;
import pl.chyla.wyjec.stdconsts.StandardConstantsParser;
import pl.chyla.wyjec.stdfuncs.StandardFunctions;
import pl.chyla.wyjec.utils.RawString;


/**
 * Command line application for evaluating Wyjec expressions and testing custom functions.
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
public final class WyjecTool {

    /**
     * Name of <code>public static</code> method defined by classes given in the command line
     * and used to produce function instances.
     */
    private static final String GET_FUNCS_METHOD_NAME = "getFunctions";

    /**
     * Parser used by the tool.
     */
    private final Parser parser;

    /**
     * Evaluator used by the tool.
     */
    private final Evaluator evaluator;

    /**
     * Output stream used for printing messages.
     */
    private final PrintStream outStream;

    /**
     * Constructs singleton <code>WyjecTool</code> instance.
     *
     * @param optsProv object that provides access to command line options
     */
    private WyjecTool(CLIOptionsProvider optsProv) throws Exception {
        Config config = Config.getDefaultConfig();
        parser = createParser(optsProv, config);
        evaluator = new Evaluator(config);
        outStream = System.out;
        String[] realArgs = optsProv.getArguments();
        if (realArgs.length > 1) {
            outStream.println(
                "Usage: <command-path> [-nostdfuncs] [-nostdconsts]"
                + " -fclass <class-name> ... -fdoc <document-path> -fexdoc <document-path>"
                + " ...");
            return;
        }
        String[] examplesDocsPaths = optsProv.getOptFuncsExamplesDocs();
        for (int i = 0; i < examplesDocsPaths.length; i++) {
            InputStream funcsDocInput =
                new BufferedInputStream(new FileInputStream(examplesDocsPaths[i]));
            FunctionsDocument funcsDoc = FunctionsDocumentParser.parseDocument(funcsDocInput);
            FunctionInformation[] funcsInfos = funcsDoc.getFunctionsInformations();
            for (int j = 0; j < funcsInfos.length; j++) {
                checkFunctionExamples(funcsInfos[j]);
            }
        }
        if (realArgs.length == 1) {
            evaluateExpression(realArgs[0]);
        }
    }

    /**
     * Creates an instance of Wyjec parser using specified options from the command line and
     * configuration object.
     *
     * @param optsProv object providing command line options
     * @param config configuration to use for the parser
     * @return
     *     new instance of Wyjec parser supporting constants and functions specified in the
     *     command line
     * @throws InvalidFactoryClassException
     *     in case of error obtaining functions from one of factory classes specified in the
     *     command line
     * @throws IOException
     *     in case of error accessing one of XML files containing descriptions of functions
     * @throws FunctionsDocumentProcessingException
     *     in case of error processing XML documents with functions descriptions
     */
    private Parser createParser(CLIOptionsProvider optsProv, Config config)
        throws InvalidFactoryClassException, IOException, FunctionsDocumentProcessingException {
        List funcs = new LinkedList();
        // functions from standard Wyjec library
        if (!optsProv.getOptNoStdFuncs()) {
            funcs.addAll(Arrays.asList(StandardFunctions.getFunctions()));
        }
        // functions implementations got from "provider" classes via static method
        String[] funcsClasses = optsProv.getOptFuncsClasses();
        for (int i = 0; i < funcsClasses.length; i++) {
            funcs.addAll(Arrays.asList(getFunctionsFromClass(funcsClasses[i])));
        }
        // simulated functions
        Function[] simulatedFuncs = SimulatedFunctionsFactory.generateFunctionsFromDocs(
            optsProv.getOptFuncsDocs(), new CLIFunctionHandler());
        funcs.addAll(Arrays.asList(simulatedFuncs));
        IConstantsParser constsParser;
        if (optsProv.getOptNoStdConsts()) {
            constsParser = EmptyConstantsParser.getInstance();
        } else {
            constsParser = StandardConstantsParser.getInstance();
        }
        Function[] funcsArray = (Function[]) funcs.toArray(new Function[0]);
        return new Parser(config, funcsArray, constsParser);
    }

    /**
     * Get an array of function instances from a class with specified name using reflection. The
     * class must have <code>public static</code> method named {@value #GET_FUNCS_METHOD_NAME}
     * that doesn't require any parameters and returns <code>Function[]</code> object. That
     * method is used to produce functions.
     *
     * @param implClassName fully-qualified name of the class to use as functions factory
     * @return an array of {@link Function} instances produced by the class
     * @throws InvalidFactoryClassException
     *     in case a class with specified name doesn't exist, doesn't have the required method,
     *     the method can't be called or it throws an exception
     */
    private Function[] getFunctionsFromClass(String implClassName)
        throws InvalidFactoryClassException {
        Method getter;
        try {
            Class cls = Class.forName(implClassName);
            getter = cls.getMethod(GET_FUNCS_METHOD_NAME, new Class[0]);
        } catch (ClassNotFoundException e) {
            throw new InvalidFactoryClassException(
                implClassName, "Can't find class with this name");
        } catch (NoSuchMethodException e) {
            throw new InvalidFactoryClassException(
                implClassName, "The class doesn't have " + GET_FUNCS_METHOD_NAME + "() method");
        } catch (SecurityException e) {
            throw new InvalidFactoryClassException(
                implClassName, "Can't access " + GET_FUNCS_METHOD_NAME + "() method");
        }
        if (getter.getReturnType().getComponentType() != Function.class
            || !Modifier.isStatic(getter.getModifiers())
            || !Modifier.isPublic(getter.getModifiers())) {
            throw new InvalidFactoryClassException(
                implClassName,
                GET_FUNCS_METHOD_NAME + "() is not public static method returning Function[]");
        }
        Object result;
        try {
            result = getter.invoke(null, new Object[] {});
        } catch (IllegalArgumentException e) {
            result = new Object[0];
            Debugging.assumeNotReached();
        } catch (IllegalAccessException e) {
            throw new InvalidFactoryClassException(
                implClassName, "Can't execute " + GET_FUNCS_METHOD_NAME + "() method");
        } catch (InvocationTargetException e) {
            throw new InvalidFactoryClassException(
                implClassName,
                "Got exception from " + GET_FUNCS_METHOD_NAME + "(): " + e.getMessage());
        }
        return (Function[]) result;
    }

    /**
     * Checks all example expressions contained in given function information, reporting status
     * to output stream.
     *
     * @param funcInfo object containing function information
     */
    private void checkFunctionExamples(FunctionInformation funcInfo) {
        outStream.println("Testing " + funcInfo.getDeclarationDescription());
        ExpressionExample[] examples = funcInfo.getExamples();
        for (int j = 0; j < examples.length; j++) {
            ExpressionExample example = examples[j];
            ExpressionTree exprTree;
            try {
                exprTree = parser.parse(new RawString(example.getExpression()));
            } catch (InvalidInputException e) {
                reportInvalidExample(
                    funcInfo, example, "Error parsing expression: " + e.getMessage());
                continue;
            }
            Value result;
            try {
                result = evaluator.evaluate(exprTree);
            } catch (EvaluationException e) {
                reportInvalidExample(
                    funcInfo, example, "Error evaluating expression: " + e.getMessage());
                continue;
            }
            if (result.getType() != example.getResultType()) {
                reportInvalidExample(
                    funcInfo, example,
                    "Invalid result type. Expected " + example.getResultType().getName()
                    + ", got " + result.getType().getName() + ".");
                continue;
            }
            if (!result.toString().equals(example.getResultString())) {
                reportInvalidExample(
                    funcInfo, example,
                    "Invalid result. Expected \"" + example.getResultString() + "\""
                    + ", got \"" + result.toString() + "\".");
            }
        }
    }

    /**
     * Prints information about invalid example expression to output stream.
     *
     * @param funcInfo information about function the example is attached to
     * @param example information about example expression
     * @param message detailed error message
     */
    private void reportInvalidExample(
        FunctionInformation funcInfo, ExpressionExample example, String message) {
        outStream.println("** Invalid example: " + example.getExpression());
        outStream.println("** " + message);
    }

    /**
     * Evaluates given expression and print evaluation results to the output stream. In case of
     * simulated functions it interactively asks for partial results.
     *
     * @param expression expression to evaluate
     */
    private void evaluateExpression(String expression) {
        RawString input = new RawString(expression);
        ExpressionTree exprTree;
        try {
            exprTree = parser.parse(input);
        } catch (InvalidInputException e) {
            outStream.println("Error parsing the expression.");
            outStream.println(expression);
            StringBuffer posPtrLine = new StringBuffer();
            for (int i = 0; i < e.getErrorPos(); i++) {
                posPtrLine.append(' ');
            }
            posPtrLine.append('^');
            outStream.println(posPtrLine);
            String msg = e.getMessage();
            if (msg != null) {
                outStream.println(msg);
            }
            exprTree = null;
        }
        if (exprTree != null) {
            try {
                Value result = evaluator.evaluate(exprTree);
                outStream.println("Result type: " + result.getType().getName());
                outStream.println("Result: " + result.toString());
            } catch (EvaluationException e) {
                outStream.println("Evaluation Error!");
                for (Throwable cause = e.getCause(); cause != null; cause = cause.getCause()) {
                    String msg = cause.getMessage();
                    if (msg != null) {
                        outStream.println(cause.getClass() + ": " + msg);
                    } else {
                        outStream.println(cause.getClass());
                    }
                }
            }
        }
    }

    /**
     * The main entry to Wyjec tool from JVM. It simply creates new instance of
     * <code>WyjecTool</code> passing given arguments to its constructor.
     *
     * @param args the program's arguments from the command line
     */
    public static void main(String[] args) throws Exception {
        new WyjecTool(new CLIOptionsProvider(args));
    }


    /**
     * Exception indicating that it's not possible to get functions from a factory class because
     * it doesn't exist, doesn't have the required static method or the method can't be executed
     * properly.
     *
     * @author Zbigniew Chyla (zbigniew@chyla.pl)
     */
    private static final class InvalidFactoryClassException extends Exception {

        /**
         * Constructs <code>InvalidFactoryClassException</code> using specified error message.
         *
         * @param factClassName name of the factory class
         * @param msg detailed error message
         */
        private InvalidFactoryClassException(String factClassName, String msg) {
            super(msg);
        }

    }

}
