/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.funcdoclet;

import java.util.LinkedList;
import java.util.List;
import com.sun.javadoc.DocErrorReporter;


/**
 * Class providing Wyjec doclet's command line options in processed form. Instances of the class
 * may be created with static factory method that parses raw command line parameters given as
 * strings.
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
final class CLIOptionsProvider {

    /**
     * Name of the command line option used to specify the title.
     */
    private static final String TITLE_OPTION = "-title";

    /**
     * Name of the command line option used to specify a subtitle (may be used multiple times).
     */
    private static final String SUBTITLE_OPTION = "-subtitle";

    /**
     * Name of the command line option used to specify the output file.
     */
    private static final String OUTPUT_OPTION = "-output";

    /**
     * Title of the document specified using {@value #TITLE_OPTION} option.
     */
    private final String title;

    /**
     * Subtitles of the document specified using {@value #SUBTITLE_OPTION} options.
     */
    private final String[] subtitles;

    /**
     * A name of the output file specified using {@value #OUTPUT_OPTION} options.
     */
    private final String output;

    /**
     * Constructs <code>CLIOptionsProvider</code> instance that holds given data parsed from
     * javadoc's command line.
     *
     * @param title the document's title
     * @param subtitles the document's subtitles
     * @param output name of the output file for the document
     */
    private CLIOptionsProvider(String title, String[] subtitles, String output) {
        this.title = title;
        this.subtitles = subtitles;
        this.output = output;
    }

    /**
     * Parses Wyjec doclet's command line options and generates the corresponding
     * {@link CLIOptionsProvider} instance that encapsulates them.
     *
     * @param options
     *    array of command line options with arguments, as provided by javadoc tool via
     *    <code>RootDoc</code>'s <code>options()</code> method
     * @param errRep object used for reporting errors and warnings during processing
     * @return
     *     {@link CLIOptionsProvider} instance containing parsed options or <code>null</code>
     *     if an error has occurred during parsing
     */
    public static CLIOptionsProvider parseOptions(String[][] options, DocErrorReporter errRep) {
        String title = null;
        List subtitlesList = new LinkedList();
        String output = null;
        boolean hasErrors = false;
        for (int i = 0; i < options.length; i++) {
            String optName = options[i][0];
            if (optName.equals(TITLE_OPTION)) {
                if (title == null) {
                    title = options[i][1];
                } else {
                    errRep.printError("Duplicate option: " + TITLE_OPTION);
                    hasErrors = true;
                }
            } else if (optName.equals(SUBTITLE_OPTION)) {
                subtitlesList.add(options[i][1]);
            } else if (optName.equals(OUTPUT_OPTION)) {
                if (output == null) {
                    output = options[i][1];
                } else {
                    errRep.printError("Duplicate option: " + OUTPUT_OPTION);
                    hasErrors = true;
                }
            }
        }
        if (title == null) {
            errRep.printError("Missing " + TITLE_OPTION + " option");
            hasErrors = true;
        }
        if (output == null) {
            errRep.printError("Missing " + OUTPUT_OPTION + " option");
            hasErrors = true;
        }
        if (!hasErrors) {
            String[] subtitles = (String[]) subtitlesList.toArray(new String[0]);
            return new CLIOptionsProvider(title, subtitles, output);
        } else {
            return null;
        }
    }

    /**
     * Method used by javadoc tool to discover a number of arguments used by given command line
     * option. It returns positive values for options recognized by Wyjec doclet and
     * <code>0</code> for every thing else.
     *
     * @param optionName name of queried option
     * @return
     *     number of arguments required for given option (including the option itself) or
     *     <code>0</code> if given option is not supported.
     */
    public static int getOptionLength(String optionName) {
        if (optionName.equals(OUTPUT_OPTION) || optionName.equals(TITLE_OPTION)
            || optionName.equals(SUBTITLE_OPTION)) {
            return 2;
        }
        return 0;
    }

    /**
     * Returns a title of the document, as specified in the command line option.
     *
     * @return document's title
     */
    public String getTitle() {
        return title;
    }

    /**
     * Returns subtitles of the document, as specified in the command line options.
     *
     * @return an array of document's subtitles (may be empty)
     */
    public String[] getSubtitles() {
        return subtitles;
    }

    /**
     * Returns a name of the output file, as specified in the command line options.
     *
     * @return name of the output file
     */
    public String getOutput() {
        return output;
    }

}
