/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.funcdoclet;

import java.io.BufferedOutputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.LinkedList;
import java.util.List;
import com.sun.javadoc.ClassDoc;
import com.sun.javadoc.DocErrorReporter;
import com.sun.javadoc.RootDoc;
import pl.chyla.wyjec.funcinfo.FunctionInformation;
import pl.chyla.wyjec.funcinfo.FunctionsDocument;
import pl.chyla.wyjec.funcinfoxml.DocumentGeneratingException;
import pl.chyla.wyjec.funcinfoxml.FunctionsDocumentGenerator;


/**
 * The main class of Wyjec Doclet implementation.
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
public final class Doclet {

    /**
     * Private empty constructor to disallow instantiating of the class.
     */
    private Doclet() { }

    /**
     * Processes program structure and command line options provided by javadoc tool and
     * generates XML document with information about Wyjec functions.
     *
     * @param root the root of the program structure provided by javadoc
     * @return
     *     <code>true</code> if the doclet has successfully written the output file (even if
     *     some errors or warnings have been printed out), <code>false</code> otherwise
     * @throws DocumentGeneratingException
     *     in case of error generating or saving the output XML file
     */
    public static boolean start(RootDoc root) throws DocumentGeneratingException {
        CLIOptionsProvider options = CLIOptionsProvider.parseOptions(root.options(), root);
        if (options == null) {
            return false;
        }
        FunctionInformation[] funcsInfos = getFuncsInfos(root.classes(), root);
        OutputStream output;
        try {
            output = new BufferedOutputStream(new FileOutputStream(options.getOutput()));
        } catch (FileNotFoundException e) {
            root.printError("Can't create output file: " + e.getMessage());
            return false;
        }
        FunctionsDocument funcsDoc = new FunctionsDocument(
            options.getTitle(), options.getSubtitles(), funcsInfos);
        try {
            FunctionsDocumentGenerator.generateDocument(funcsDoc, output);
        } finally {
            try {
                output.close();
            } catch (IOException e) {
                root.printError("Error closing output file: " + e.getMessage());
                return false;
            }
        }
        return true;
    }

    /**
     * Processes descriptions of classes provided by javadoc and generates descriptions of Wyjec
     * functions using comments attached to classes that implement them.
     *
     * @param classesDocs
     *     array of <code>ClassDoc</code> instances describing classes parsed by javadoc tool
     * @param errRep object used for reporting errors and warnings during processing
     * @return
     *     array of {@link FunctionInformation} instances containing information about Wyjec
     *     functions that could be read from comments attached to classes and their fields
     */
    private static FunctionInformation[] getFuncsInfos(
        ClassDoc[] classesDocs, DocErrorReporter errRep) {
        FunctionDocParser docParser = new FunctionDocParser(errRep);
        List funcsInfosList = new LinkedList();
        FunctionInformation funcInfo;
        for (int i = 0; i < classesDocs.length; i++) {
            funcInfo = docParser.parseFunctionDoc(classesDocs[i]);
            if (funcInfo != null) {
                funcsInfosList.add(funcInfo);
            }
        }
        return (FunctionInformation[]) funcsInfosList.toArray(new FunctionInformation[0]);
    }

    /**
     * Returns the number of arguments required by specified option counting the option itself
     * as an argument. This method is a part of the API used by javadoc tool.
     *
     * @param optionName name of an option to check
     * @return number of arguments required to use this option
     */
    public static int optionLength(String optionName) {
        return CLIOptionsProvider.getOptionLength(optionName);
    }

}
