/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.funcdoclet;

import java.util.LinkedList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import com.sun.javadoc.ClassDoc;
import com.sun.javadoc.DocErrorReporter;
import com.sun.javadoc.FieldDoc;
import com.sun.javadoc.Tag;
import pl.chyla.wyjec.core.Type;
import pl.chyla.wyjec.funcinfo.ExpressionExample;
import pl.chyla.wyjec.funcinfo.FunctionInformation;
import pl.chyla.wyjec.funcinfo.ParameterInformation;
import pl.chyla.wyjec.reflection.FunctionSignatureReflection;
import pl.chyla.wyjec.reflection.ReflectiveFunction;
import pl.chyla.wyjec.reflection.TypeReflection;
import pl.chyla.wyjec.utils.xml.StandardXMLReferencesDecoder;


/**
 * Parser for javadoc comments describing Wyjec function and its parameters. It processes
 * <code>ClassDoc</code> instance provided by javadoc tool and generates a corresponding
 * {@link FunctionInformation} instance that describes Wyjec function.
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
final class FunctionDocParser {

    /**
     * Compiled regular expression that matches text containing example of a functions usage.
     * It contains three groups that match: expression, result, result's type.
     */
    private static final Pattern WYJEC_EXAMPLE_TEXT_PATTERN =
        Pattern.compile("\\s*\"([^\"]*)\"\\s+\"([^\"]*)\"\\s+([^\\s]+)\\s*");

    /**
     * Name of javadoc tag used to mark a Wyjec function.
     */
    private static final String FUNCTION_TAG_NAME = "@wyjec.func";

    /**
     * Name of javadoc tag used to mark a Wyjec parameter.
     */
    private static final String PARAMETER_TAG_NAME = "@wyjec.param";

    /**
     * Name of javadoc tag used to mark an example of Wyjec function call.
     */
    private static final String EXAMPLE_TAG_NAME = "@wyjec.example";

    /**
     * Object used for reporting errors and warnings during processing.
     */
    private final DocErrorReporter errRep;

    /**
     * Name of parsed name of the function
     */
    private String funcName;

    /**
     * Parsed description of the function
     */
    private String funcDesciption;

    /**
     * Objects containing examples of function usage.
     */
    private ExpressionExample[] funcExamples;

    /**
     * Recently parsed example of function usage.
     */
    private ExpressionExample curExample;

    /**
     * Objects containing parsed information about required parameters of the function
     */
    private ParameterInformation[] reqParamsInfos;

    /**
     * Object containing parsed information about optional parameters of the function
     * (<code>null</code> if not available)
     */
    private ParameterInformation optParamsInfo;

    /**
     * Description of the recently parsed parameter.
     */
    private String curParamName;

    /**
     * Description of the recently parsed parameter.
     */
    private String curParamDescription;

    /**
     * Boolean flag informing whether an error has been detected in processed javadoc comments.
     */
    private boolean hasErrors;

    /**
     * Constructs <code>FunctionDocParser</code> instance using given object for reporting
     * detected errors and warnings.
     *
     * @param errRep object used for reporting errors and warnings during processing
     */
    public FunctionDocParser(DocErrorReporter errRep) {
        this.errRep = errRep;
    }

    /**
     * Processes object representing a class that implements Wyjec function and generates
     * {@link FunctionInformation} instance that describes this function.
     *
     * @param clsDoc object representing a class to process
     * @return
     *     object containing information about Wyjec function implemented by the class or
     *     <code>null</code> if an error occurred during processing
     */
    public FunctionInformation parseFunctionDoc(ClassDoc clsDoc) {
        Type funcResType = getFunctionResultType(clsDoc);
        if (funcResType == null) {
            return null;
        }
        hasErrors = false;
        processClassDoc(clsDoc);
        processFieldsDocs(clsDoc.fields());
        FunctionInformation funcInfo;
        if (!hasErrors) {
            funcInfo = new FunctionInformation(
                funcName, funcResType, reqParamsInfos, optParamsInfo, funcDesciption,
                funcExamples);
        } else {
            funcInfo = null;
        }
        return funcInfo;
    }

    /**
     * Returns type of Wyjec function implemented by a class described by given object.
     *
     * @param clsDoc object representing a class to analyse
     * @return detected return type of the function
     */
    private Type getFunctionResultType(ClassDoc clsDoc) {
        String clsName = clsDoc.qualifiedName();
        Type funcResType = ReflectiveFunction.getResultTypeByFunctionClassName(clsName);
        if (funcResType == null) {
            ClassDoc superClsDoc = clsDoc.superclass();
            if (superClsDoc != null) {
                funcResType = getFunctionResultType(superClsDoc);
            }
        }
        return funcResType;
    }

    /**
     * Parses documentation string of given class that implements Wyjec function and checks if
     * it's tagged correctly. In case of success, the function's name gets copied to
     * {@link #funcName} and its description gets copied to {@link #funcDesciption}. In
     * case of detecting any errors the method sets {@link #hasErrors} to <code>true</code>.
     *
     * @param clsDoc object representing Java class
     */
    private void processClassDoc(ClassDoc clsDoc) {
        List examples = new LinkedList();
        funcName = null;
        Tag[] tags = clsDoc.tags();
        for (int i = 0; i < tags.length; i++) {
            Tag tag = tags[i];
            if (tag.name().equals(EXAMPLE_TAG_NAME)) {
                processExample(tag);
                if (curExample != null) {
                    examples.add(curExample);
                }
            } else if (tag.name().equals(FUNCTION_TAG_NAME)) {
                String tagText = tag.text().trim();
                if (tagText.length() == 0) {
                    errRep.printError(tag.position(), "Empty function's name");
                    hasErrors = true;
                } else if (funcName != null) {
                    errRep.printError(
                        tag.position(), "Duplicate " + FUNCTION_TAG_NAME + " tag for function");
                    hasErrors = true;
                } else {
                    funcName = tagText;
                }
            }
        }
        if (funcName == null) {
            errRep.printError(
                clsDoc.position(), "Missing " + FUNCTION_TAG_NAME + " tag for function");
            hasErrors = true;
        }
        funcDesciption = processDescription(clsDoc.commentText());
        funcExamples = (ExpressionExample[]) examples.toArray(new ExpressionExample[0]);
    }

    /**
     * Parses example of function usage contained in given {@value #EXAMPLE_TAG_NAME} tag. In
     * case of success, an object representing the example gets copied to {@link #curExample}.
     * In case of detecting any errors the method sets {@link #hasErrors} to <code>true</code>.
     *
     * @param tag tag containing the example to parse
     */
    private void processExample(Tag tag) {
        String text = tag.text().trim();
        Matcher m = WYJEC_EXAMPLE_TEXT_PATTERN.matcher(text);
        if (m.matches()) {
            String expression = StandardXMLReferencesDecoder.decode(m.group(1));
            String resultString = StandardXMLReferencesDecoder.decode(m.group(2));
            String typeName = m.group(3);
            Type resultType = TypeReflection.getTypeByName(typeName);
            if (resultType == null) {
                errRep.printError(
                    tag.position(), "Invalid type of the result: " + typeName);
                hasErrors = true;
            }
            if (!hasErrors) {
                curExample = new ExpressionExample(expression, resultType, resultString);
            }
        } else {
            errRep.printError(
                tag.position(), "Invalid syntax of " + EXAMPLE_TAG_NAME + " tag."
                + " Should be: " + EXAMPLE_TAG_NAME + " \"<expression>\" \"<result>\" <type>");
            hasErrors = true;
        }
    }

    /**
     * Processes an array of fields by creating {@link ParameterInformation} instance for every
     * field recognized as Wyjec parameter and storing an array of these instances in
     * {@link #reqParamsInfos} field. In case of detecting any errors the method sets
     * {@link #hasErrors} to <code>true</code>.
     *
     * @param fieldsDocs array of objects representing fields in Java class
     */
    private void processFieldsDocs(FieldDoc[] fieldsDocs) {
        List reqParamsInfosList = new LinkedList();
        optParamsInfo = null;
        for (int i = 0; i < fieldsDocs.length; i++) {
            FieldDoc fieldDoc = fieldsDocs[i];
            String fieldClsName = fieldDoc.type().qualifiedTypeName();
            Type reqParamType =
                FunctionSignatureReflection.getParameterTypeByClassName(fieldClsName);
            if (reqParamType != null) {
                processFieldDoc(fieldDoc);
                if (!hasErrors) {
                    reqParamsInfosList.add(new ParameterInformation(
                        curParamName, reqParamType, curParamDescription));
                }
            } else {
                Type optParamsType =
                    FunctionSignatureReflection.getOptParametersTypeByClassName(fieldClsName);
                if (optParamsType != null) {
                    if (optParamsInfo == null) {
                        processFieldDoc(fieldDoc);
                        if (!hasErrors) {
                            optParamsInfo = new ParameterInformation(
                                curParamName, optParamsType, curParamDescription);
                        }
                    } else {
                        errRep.printError(
                            fieldDoc.position(),
                            "Duplicate declaration of function's optional parameters");
                        hasErrors = true;
                    }
                }
            }
        }
        reqParamsInfos =
            (ParameterInformation[]) reqParamsInfosList.toArray(new ParameterInformation[0]);
    }

    /**
     * Parses documentation string of given field that represents Wyjec parameter and checks if
     * it's tagged correctly. In case of success, the parameter's name gets copied to
     * {@link #curParamName} and its description gets copied to {@link #curParamDescription}. In
     * case of detecting any errors the method sets {@link #hasErrors} to <code>true</code>.
     *
     * @param fieldDoc object representing a field in Java class
     */
    private void processFieldDoc(FieldDoc fieldDoc) {
        curParamName = null;
        Tag[] tags = fieldDoc.tags();
        for (int i = 0; i < tags.length; i++) {
            Tag tag = tags[i];
            if (tag.name().equals(PARAMETER_TAG_NAME)) {
                String tagText = tag.text().trim();
                if (tagText.length() == 0) {
                    errRep.printError(tag.position(), "Empty parameter's name");
                    hasErrors = true;
                } else if (curParamName != null) {
                    errRep.printError(
                        tag.position(),
                        "Duplicate " + PARAMETER_TAG_NAME + " tag for parameter");
                    hasErrors = true;
                } else {
                    curParamName = tagText;
                }
            }
        }
        if (curParamName == null) {
            errRep.printError(
                fieldDoc.position(), "Missing " + PARAMETER_TAG_NAME + " tag for parameter");
            hasErrors = true;
        }
        curParamDescription = processDescription(fieldDoc.commentText());
    }

    /**
     * Processes description of function or parameter by removing end-of-line characters and
     * removing spaces at the beginning and end of every line.
     *
     * @param desc the original description, as stored in a comment
     * @return processed description, as a single line of text
     */
    private String processDescription(String desc) {
        int descLen = desc.length();
        StringBuffer result = new StringBuffer();
        int idx = 0;
        while (idx < descLen) {
            int eolPos = desc.indexOf('\n', idx);
            if (eolPos < 0) {
                eolPos = descLen;
            }
            String line = desc.substring(idx, eolPos).trim();
            if (result.length() > 0) {
                result.append(' ');
            }
            result.append(line);
            idx = eolPos + 1;
        }
        return StandardXMLReferencesDecoder.decode(result.toString());
    }

}
