/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.funcinfo;

import pl.chyla.wyjec.core.Type;


/**
 * Contains all information about single Wyjec function except its implementation: function's
 * name and description, type of result, information about required and optional parameters
 * (both using references to objects describing them).
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
public final class FunctionInformation {

    /**
     * Function's name
     */
    private final String name;

    /**
     * Type of function's result
     */
    private final Type resultType;

    /**
     * Array of objects containing information about function's declared required parameters
     */
    private final ParameterInformation[] reqParams;

    /**
     * Information about function's declared optional parameters or <code>null</code> if this
     * function has no optional parameters declared
     */
    private final ParameterInformation optParams;

    /**
     * Human-readable function's description
     */
    private final String description;

    /**
     * Examples of the function's usage.
     */
    private final ExpressionExample[] examples;

    /**
     * Constructs <code>FunctionInformation</code> instance describing Wyjec function with given
     * properties.
     *
     * @param name function's name
     * @param resultType function's result type
     * @param reqParams function's required parameters
     * @param optParams
     *     object describing function's optional parameters or <code>null</code> if this
     *     function doesn't accept optional parameters
     * @param description
     *     function's description or <code>null</code> if the description is not available
     * @param examples array of objects containing examples of the function usage
     */
    public FunctionInformation(
        String name, Type resultType, ParameterInformation[] reqParams,
        ParameterInformation optParams, String description, ExpressionExample[] examples) {
        if (name == null || resultType == null || reqParams == null || examples == null) {
            throw new IllegalArgumentException(
                "Functions's name, result type, required parameters and array of examples"
                + "must not be null");
        }
        this.name = name;
        this.resultType = resultType;
        this.reqParams = reqParams;
        this.optParams = optParams;
        this.description = description;
        this.examples = examples;
    }

    /**
     * Returns name of this function.
     *
     * @return this function's name
     */
    public String getName() {
        return name;
    }

    /**
     * Returns type of values returned by this function.
     *
     * @return this function's result type
     */
    public Type getResultType() {
        return resultType;
    }

    /**
     * Returns information about required parameters of this function.
     *
     * @return
     *     this function's required parameters information; the returned array should not be
     *     modified
     */
    public ParameterInformation[] getRequiredParameters() {
        return reqParams;
    }

    /**
     * Returns information about optional parameters of this function. The information is
     * presented in single {@link ParameterInformation} instance, because all optional
     * parameters have the same properties.
     *
     * @return
     *     this function's optional parameters information or <code>null</code> if this function
     *     doesn't accept any optional parameters
     */
    public ParameterInformation getOptionalParameters() {
        return optParams;
    }

    /**
     * Returns human-readable description of this function.
     *
     * @return this function's description
     */
    public String getDescription() {
        return description;
    }

    /**
     * Returns examples demonstrating how the function may be used.
     *
     * @return
     *     array of objects containing examples of the function usage; the returned array should
     *     not be modified
     */
    public ExpressionExample[] getExamples() {
        return examples;
    }

    /**
     * Returns description of the function's name and signature as single string value
     * containing: the function name, names and types of parameters and the result type.
     *
     * @return string containing description of the function
     */
    public String getDeclarationDescription() {
        StringBuffer desc = new StringBuffer();
        desc.append(getName());
        desc.append(" (");
        ParameterInformation[] reqParamsInfos = getRequiredParameters();
        for (int i = 0; i < reqParamsInfos.length; i++) {
            if (i > 0) {
                desc.append(", ");
            }
            desc.append(reqParamsInfos[i].getName());
            desc.append(" : ");
            desc.append(reqParamsInfos[i].getType().getName());
        }
        ParameterInformation optParamsInfo = getOptionalParameters();
        if (optParamsInfo != null) {
            if (reqParamsInfos.length != 0) {
                desc.append(", ");
            }
            desc.append(optParamsInfo.getName());
            desc.append(" : ");
            desc.append(optParamsInfo.getType().getName() + "...");
        }
        desc.append(") : ");
        desc.append(getResultType().getName());
        return desc.toString();
    }

}
