/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.funcinfo;


/**
 * Contains all information about a document describing a group of Wyjec functions: document's
 * title, optional subtitles and a sequence of function information (name, signature, etc.).
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
public final class FunctionsDocument {

    /**
     * This document's title
     */
    private final String title;

    /**
     * Array of This document's subtitles (may be empty)
     */
    private final String[] subtitles;

    /**
     * Array of references to objects describing functions contained in this document (may be
     * empty)
     */
    private final FunctionInformation[] funcsInfos;

    /**
     * Constructs <code>FunctionsDocument</code> instance describing document with spcified
     * information about Wyjec functions.
     *
     * @param title document's title
     * @param subtitles document's subtitles
     * @param funcsInfos information about functions contained in the document
     * @throws IllegalArgumentException
     *     in case any parameter of the constructor is <code>null</code>
     */
    public FunctionsDocument(
        String title, String[] subtitles, FunctionInformation[] funcsInfos) {
        if (title == null || subtitles == null || funcsInfos == null) {
            throw new IllegalArgumentException(
                "Title, subtitles and funcsInfos must not be null");
        }
        this.title = title;
        this.subtitles = subtitles;
        this.funcsInfos = funcsInfos;
    }

    /**
     * Returns the title of this document.
     *
     * @return this document's title
     */
    public String getTitle() {
        return title;
    }

    /**
     * Returns (possibly empty) array of subtitles of this document.
     *
     * @return this document's subtitles; the returned array should not be modified
     */
    public String[] getSubtitles() {
        return subtitles;
    }

    /**
     * Returns (possibly empty) array of information about functions contained in this document.
     *
     * @return functions contained in this document; the returned array should not be modified
     */
    public FunctionInformation[] getFunctionsInformations() {
        return funcsInfos;
    }

}
