/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.funcinfo;

import pl.chyla.wyjec.core.Type;


/**
 * Contains all information about single required parameter of Wyjec function or about single
 * group of a function's optional parameters. In case of single parameter it covers its name,
 * type and description. In case of a group of optional parameters the name and the description
 * are bound to all optional parameters as the whole, while the type specifies a type of every
 * optional parameter in the group.
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
public final class ParameterInformation {

    /**
     * This parameter's name.
     */
    private final String name;

    /**
     * This parameter's type.
     */
    private final Type type;

    /**
     * This parameter's human-readable description.
     */
    private final String description;

    /**
     * Constructs <code>ParameterInformation</code> instance describing Wyjec function parameter
     * with given properties.
     *
     * @param name parameter's name
     * @param type parameter's type
     * @param description
     *     parameter's description or <code>null</code> if the description is not available
     * @throws IllegalArgumentException
     *     in case given <code>name</code> or <code>type</code> is <code>null</code>
     */
    public ParameterInformation(String name, Type type, String description) {
        if (name == null || type == null) {
            throw new IllegalArgumentException("Parameter's name and type must not be null");
        }
        this.name = name;
        this.type = type;
        this.description = description;
    }

    /**
     * Returns the name of this parameter (in case of instance representing single required
     * parameter) or or this group of parameters (in case of instance representing a group of
     * optional parameters).
     *
     * @return this parameter's name
     */
    public String getName() {
        return name;
    }

    /**
     * Returns the type of this parameter (in case of instance representing single required
     * parameter) or every parameter in this group (in case of instance representing a group of
     * optional parameters).
     *
     * @return this parameter's type
     */
    public Type getType() {
        return type;
    }

    /**
     * Returns human-readable description of this parameter (in case of instance representing
     * single required parameter) or this group of parameters (in case of instance representing
     * a group of optional parameters).
     *
     * @return
     *     this parameter's description or <code>null</code> if the description is not available
     */
    public String getDescription() {
        return description;
    }

}
