/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.funcinfoxml;

import org.xml.sax.Attributes;
import pl.chyla.wyjec.core.Type;
import pl.chyla.wyjec.funcinfo.ExpressionExample;
import pl.chyla.wyjec.reflection.TypeReflection;
import pl.chyla.wyjec.utils.xml.ComplexElementHandler;
import pl.chyla.wyjec.utils.xml.ContentElementHandler;
import pl.chyla.wyjec.utils.xml.ElementHandler;
import pl.chyla.wyjec.utils.xml.IComplexElementListener;
import pl.chyla.wyjec.utils.xml.IContentElementListener;


/**
 * Handler for "Example" elements, providing new {@link ExpressionExample} instance after XML
 * parsing.
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
final class ExampleHandler extends ComplexElementHandler {

    /**
     * Value of "resultType" attribute resolved to {@link Type} instance
     */
    private Type resultType;

    /**
     * Content of "Expression" sub-element
     */
    private String expression;

    /**
     * Content of "Result" sub-element
     */
    private String resultString;

    /**
     * Handler used for processing "Expression" sub-element
     */
    private final ContentElementHandler expressionHandler = new ContentElementHandler(
        "Expression", new IContentElementListener() {
            public void elementLeaved(String elemContent) {
                expression = elemContent;
            }
        });

    /**
     * Handler used for processing "Result" sub-element
     */
    private final ContentElementHandler resultHandler = new ContentElementHandler(
        "Result", new IContentElementListener() {
            public void elementLeaved(String elemContent) {
                resultString = elemContent;
            }
        });

    /**
     * Constructs <code>ExampleHandler</code> instance.
     *
     * @param elemLeavedListener listener to activate after parsing this element
     */
    ExampleHandler(IComplexElementListener elemLeavedListener) {
        super("Example", elemLeavedListener);
    }

    /**
     * {@inheritDoc}
     */
    protected ElementHandler[] getSubHandlers() {
        return new ElementHandler[] {expressionHandler, resultHandler};
    }

    /**
     * {@inheritDoc}
     */
    protected void enter(Attributes attrs) {
        expression = null;
        String resultTypeName = attrs.getValue("resultType");
        if (resultTypeName != null) {
            resultType = TypeReflection.getTypeByName(resultTypeName);
        } else {
            resultType = null;
        }
        resultString = null;
    }

    /**
     * Returns new {@link ExpressionExample} instance corresponding to XML element processed by
     * this handler.
     *
     * @return information about processed "Example" element
     * @throws FunctionsDocumentProcessingException
     *     in case the handler detected semantic errors during processing the XML document
     */
    ExpressionExample getExpressionExample() {
        if (resultType == null) {
            throw new FunctionsDocumentProcessingException(
                "Missing \"resultType\" attribute for " + getElementName() + " element.");
        }
        if (expression == null) {
            throw new FunctionsDocumentProcessingException(
                "Missing \"Expression\" element inside " + getElementName() + ".");
        }
        if (resultString == null) {
            throw new FunctionsDocumentProcessingException(
                "Missing \"Result\" element inside " + getElementName() + ".");
        }
        return new ExpressionExample(expression, resultType, resultString);
    }

}
