/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.funcinfoxml;

import java.util.LinkedList;
import java.util.List;
import org.xml.sax.Attributes;
import pl.chyla.wyjec.core.Type;
import pl.chyla.wyjec.funcinfo.ExpressionExample;
import pl.chyla.wyjec.funcinfo.FunctionInformation;
import pl.chyla.wyjec.funcinfo.ParameterInformation;
import pl.chyla.wyjec.reflection.TypeReflection;
import pl.chyla.wyjec.utils.xml.ComplexElementHandler;
import pl.chyla.wyjec.utils.xml.ContentElementHandler;
import pl.chyla.wyjec.utils.xml.ElementHandler;
import pl.chyla.wyjec.utils.xml.IComplexElementListener;
import pl.chyla.wyjec.utils.xml.IContentElementListener;


/**
 * Handler for "Function" element, providing new {@link FunctionInformation} instance after XML
 * parsing.
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
final class FunctionHandler extends ComplexElementHandler {

    /**
     * Value of "name" attribute
     */
    private String name;

    /**
     * Value of "resultType" attribute resolved to {@link Type} instance
     */
    private Type resultType;

    /**
     * Content of "Description" sub-element
     */
    private String description;

    /**
     * List of {@link ParameterInformation} instances representing contents of
     * "RequiredParameter" elements
     */
    private final List reqParamsInfos = new LinkedList();

    /**
     * {@link ParameterInformation} instance representing content of "OptionalParameters"
     * element
     */
    private ParameterInformation optParamsInfo;

    /**
     * List of {@link ExpressionExample} instances representing contents of "Example" elements.
     */
    private final List examples = new LinkedList();

    /**
     * Handler used for processing "Description" sub-element
     */
    private final ContentElementHandler descriptionHandler = new ContentElementHandler(
        "Description", new IContentElementListener() {
            public void elementLeaved(String elemContent) {
                description = elemContent;
            }
        });

    /**
     * Handler used for processing "RequiredParameter" sub-elements
     */
    private final ParameterHandler requiredParameterHandler = new ParameterHandler(
        "RequiredParameter", new IComplexElementListener() {
            public void elementLeaved() {
                reqParamsInfos.add(requiredParameterHandler.getParameterInformation());
            }
        });

    /**
     * Handler used for processing "OptionalParameters" sub-element
     */
    private final ParameterHandler optionalParametersHandler = new ParameterHandler(
        "OptionalParameters", new IComplexElementListener() {
            public void elementLeaved() {
                optParamsInfo = optionalParametersHandler.getParameterInformation();
            }
        });

    /**
     * Handler used for processing "Example" sub-elements
     */
    private final ExampleHandler exampleHandler = new ExampleHandler(
        new IComplexElementListener() {
            public void elementLeaved() {
                examples.add(exampleHandler.getExpressionExample());
            }
        });

    /**
     * Constructs <code>FunctionHandler</code> instance.
     *
     * @param elemLeavedListener listener to activate after parsing this element
     */
    FunctionHandler(IComplexElementListener elemLeavedListener) {
        super("Function", elemLeavedListener);
    }

    /**
     * {@inheritDoc}
     */
    protected ElementHandler[] getSubHandlers() {
        return new ElementHandler[] {
            requiredParameterHandler, optionalParametersHandler, descriptionHandler,
            exampleHandler};
    }

    /**
     * {@inheritDoc}
     */
    protected void enter(Attributes attrs) {
        name = attrs.getValue("name");
        String resultTypeName = attrs.getValue("resultType");
        if (resultTypeName != null) {
            resultType = TypeReflection.getTypeByName(resultTypeName);
        } else {
            resultType = null;
        }
        description = null;
        reqParamsInfos.clear();
        optParamsInfo = null;
        examples.clear();
    }

    /**
     * Returns new {@link FunctionInformation} instance corresponding to XML element processed
     * by this handler.
     *
     * @return information about processed "Function" element
     * @throws FunctionsDocumentProcessingException
     *     in case the handler detected semantic errors during processing the XML document
     */
    FunctionInformation getFunctionInformation() {
        if (name == null) {
            throw new FunctionsDocumentProcessingException(
                "Missing \"name\" attribute for " + getElementName() + " element.");
        }
        if (resultType == null) {
            throw new FunctionsDocumentProcessingException(
                "Missing \"resultType\" attribute for " + getElementName() + " element.");
        }
        ParameterInformation[] reqParamsInfosArray =
            (ParameterInformation[]) reqParamsInfos.toArray(new ParameterInformation[0]);
        ExpressionExample[] examplesArray =
            (ExpressionExample[]) examples.toArray(new ExpressionExample[0]);
        return new FunctionInformation(
            name, resultType, reqParamsInfosArray, optParamsInfo, description, examplesArray);
    }

}
