/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.funcinfoxml;

import java.io.OutputStream;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.sax.SAXTransformerFactory;
import javax.xml.transform.sax.TransformerHandler;
import javax.xml.transform.stream.StreamResult;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

import pl.chyla.wyjec.funcinfo.ExpressionExample;
import pl.chyla.wyjec.funcinfo.FunctionInformation;
import pl.chyla.wyjec.funcinfo.FunctionsDocument;
import pl.chyla.wyjec.funcinfo.ParameterInformation;


/**
 * Utility class for storing information described by <code>FunctionsDocument</code> in any data
 * stream as XML document. The DTD of used document's format is available at
 * <a href="http://zbigniew.chyla.pl/wyjec/functions-document-1.0.dtd">
 * http://zbigniew.chyla.pl/wyjec/functions-document-1.0.dtd</a>.
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
public final class FunctionsDocumentGenerator {

    /**
     * URI of the DTD defining format of XML documents generated by this class.
     */
    public static final String FUNCTIONS_DOCUMENT_DTD_URI =
        "http://zbigniew.chyla.pl/wyjec/functions-document-1.0.dtd";

    /**
     * Private empty constructor to disallow instantiating of the class.
     */
    private FunctionsDocumentGenerator() { }

    /**
     * Generates XML document representing information stored in given {@link FunctionsDocument}
     * instance.
     *
     * @param funcsDoc
     *     {@link FunctionsDocument} instance used as the source of data for generated XML
     *     document
     * @param output output stream to write generated XML document to
     * @throws DocumentGeneratingException in case of error generating or saving the document
     */
    public static void generateDocument(FunctionsDocument funcsDoc, OutputStream output)
        throws DocumentGeneratingException {
        TransformerHandler th;
        try {
            th = getTransformerHandler(output);
        } catch (TransformerConfigurationException e) {
            throw new DocumentGeneratingException(
                "Can't obtain TransformerHandler instance", e);
        }
        try {
            th.startDocument();
            th.startElement("", "", "FunctionsDocument", new AttributesImpl());
            String title = funcsDoc.getTitle();
            if (title != null) {
                th.startElement("", "", "Title", new AttributesImpl());
                th.characters(title.toCharArray(), 0, title.length());
                th.endElement(null, "", "Title");
            }
            String[] subtitles = funcsDoc.getSubtitles();
            for (int i = 0; i < subtitles.length; i++) {
                String subtitle = subtitles[i];
                th.startElement("", "", "Subtitle", new AttributesImpl());
                th.characters(subtitle.toCharArray(), 0, subtitle.length());
                th.endElement(null, "", "Subtitle");
            }
            FunctionInformation[] funcsInfos = funcsDoc.getFunctionsInformations();
            for (int i = 0; i < funcsInfos.length; i++) {
                genFunctionElem(funcsInfos[i], th);
            }
            th.endElement(null, "", "FunctionsDocument");
            th.endDocument();
        } catch (SAXException e) {
            throw new DocumentGeneratingException("Can't write XML data", e);
        }
    }

    /**
     * Obtains <code>TransformerHandler</code> instance to use for generating XML data that is
     * to be written to given output stream.
     *
     * @param output output stream to write generated XML document to
     * @return obtained <code>TransformerHandler</code> instance
     * @throws TransformerConfigurationException
     *    in case of configuration error making it impossible to get transformer handler
     */
    private static TransformerHandler getTransformerHandler(OutputStream output)
        throws TransformerConfigurationException {
        TransformerFactory transFact = TransformerFactory.newInstance();
        if (!transFact.getFeature(SAXTransformerFactory.FEATURE)) {
            throw new TransformerConfigurationException(
                "Can't get an instance of SAXTransformerFactory.");
        }
        SAXTransformerFactory saxTransFact = (SAXTransformerFactory) transFact;
        TransformerHandler th = saxTransFact.newTransformerHandler();
        Transformer trans = th.getTransformer();
        trans.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
        trans.setOutputProperty(OutputKeys.INDENT, "yes");
        trans.setOutputProperty(OutputKeys.STANDALONE, "yes");
        trans.setOutputProperty(OutputKeys.DOCTYPE_SYSTEM, FUNCTIONS_DOCUMENT_DTD_URI);
        th.setResult(new StreamResult(output));
        return th;
    }

    /**
     * Generates a part of the XML document corresponding to given function information.
     *
     * @param funcInfo
     *     {@link FunctionInformation} instance used as the source of data for generated part of
     *     the XML document
     * @param th <code>TransformerHandler</code> instance to use to generate XML document
     * @throws SAXException in case the used <code>TransformerHandler</code> throws an exception
     */
    private static void genFunctionElem(FunctionInformation funcInfo, TransformerHandler th)
        throws SAXException {
        AttributesImpl funcAttrs = new AttributesImpl();
        funcAttrs.addAttribute("", "", "name", "", funcInfo.getName());
        funcAttrs.addAttribute("", "", "resultType", "", funcInfo.getResultType().getName());
        th.startElement("", "", "Function", funcAttrs);
        String description = funcInfo.getDescription();
        if (description != null) {
            genDescription(description, th);
        }
        ParameterInformation[] reqParamsDescs = funcInfo.getRequiredParameters();
        for (int i = 0; i < reqParamsDescs.length; i++) {
            genParameterElem(reqParamsDescs[i], "RequiredParameter", th);
        }
        ParameterInformation optParamsDesc = funcInfo.getOptionalParameters();
        if (optParamsDesc != null) {
            genParameterElem(optParamsDesc, "OptionalParameters", th);
        }
        ExpressionExample[] examples = funcInfo.getExamples();
        for (int i = 0; i < examples.length; i++) {
            genExampleElem(examples[i], th);
        }
        th.endElement(null, "", "Function");
    }

    /**
     * Generates a part of the XML document corresponding to given parameter information.
     *
     * @param paramInfo
     *     {@link ParameterInformation} instance used as the source of data for generated part
     *     of the XML document
     * @param elemName the name of XML element to generate
     * @param th <code>TransformerHandler</code> instance to use to generate XML document
     * @throws SAXException in case the used <code>TransformerHandler</code> throws an exception
     */
    private static void genParameterElem(
        ParameterInformation paramInfo, String elemName, TransformerHandler th)
        throws SAXException {
        AttributesImpl paramAttrs = new AttributesImpl();
        paramAttrs.addAttribute("", "", "name", "", paramInfo.getName());
        paramAttrs.addAttribute("", "", "type", "", paramInfo.getType().getName());
        th.startElement("", "", elemName, paramAttrs);
        String description = paramInfo.getDescription();
        if (description != null) {
            genDescription(description, th);
        }
        th.endElement(null, "", elemName);
    }

    /**
     * Generates a part of XML document containing "Description" element together with contained
     * character data.
     *
     * @param description character data to put inside the element
     * @param th <code>TransformerHandler</code> instance to use to generate XML document
     * @throws SAXException in case the used <code>TransformerHandler</code> throws an exception
     */
    private static void genDescription(String description, TransformerHandler th)
        throws SAXException {
        th.startElement("", "", "Description", new AttributesImpl());
        char[] descChars = description.toCharArray();
        th.characters(descChars, 0, descChars.length);
        th.endElement(null, "", "Description");
    }

    /**
     * Generates a part of XML document containing "Example" element together with contained
     * elements ("Expression" and "Result").
     *
     * @param example
     *     {@link ExpressionExample} instance used as the source of data for generated part of
     *     the XML document
     * @param th <code>TransformerHandler</code> instance to use to generate XML document
     * @throws SAXException in case the used <code>TransformerHandler</code> throws an exception
     */
    private static void genExampleElem(ExpressionExample example, TransformerHandler th)
        throws SAXException {
        AttributesImpl exampleAttrs = new AttributesImpl();
        exampleAttrs.addAttribute("", "", "resultType", "", example.getResultType().getName());
        th.startElement("", "", "Example", exampleAttrs);
        th.startElement("", "", "Expression", new AttributesImpl());
        char[] exprChars = example.getExpression().toCharArray();
        th.characters(exprChars, 0, exprChars.length);
        th.endElement(null, "", "Expression");
        th.startElement("", "", "Result", new AttributesImpl());
        char[] resultChars = example.getResultString().toCharArray();
        th.characters(resultChars, 0, resultChars.length);
        th.endElement(null, "", "Result");
        th.endElement(null, "", "Example");
    }

}
