/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.funcinfoxml;

import java.util.LinkedList;
import java.util.List;
import org.xml.sax.Attributes;
import pl.chyla.wyjec.funcinfo.FunctionInformation;
import pl.chyla.wyjec.funcinfo.FunctionsDocument;
import pl.chyla.wyjec.utils.xml.ComplexElementHandler;
import pl.chyla.wyjec.utils.xml.ContentElementHandler;
import pl.chyla.wyjec.utils.xml.ElementHandler;
import pl.chyla.wyjec.utils.xml.IComplexElementListener;
import pl.chyla.wyjec.utils.xml.IContentElementListener;


/**
 * Handler for "FunctionsDocument" element, providing new {@link FunctionsDocument} instance
 * after XML parsing.
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
final class FunctionsDocumentHandler extends ComplexElementHandler {

    /**
     * Content of "Title" sub-element
     */
    private String title;

    /**
     * Contents of "Subtitle" sub-elements
     */
    private final List subtitles = new LinkedList();

    /**
     * List of {@link FunctionInformation} instances generated from "Function" sub-elements
     */
    private final List funcsInfos = new LinkedList();

    /**
     * Handler used for processing "Title" sub-element
     */
    private final ContentElementHandler titleHandler = new ContentElementHandler(
        "Title", new IContentElementListener() {
            public void elementLeaved(String elemContent) {
                title = elemContent;
            }
        });

    /**
     * Handler used for processing "Subtitle" sub-element
     */
    private final ContentElementHandler subTitleHandler = new ContentElementHandler(
        "Subtitle", new IContentElementListener() {
            public void elementLeaved(String elemContent) {
                subtitles.add(elemContent);
            }
        });

    /**
     * Handler used for processing "Function" sub-elements
     */
    private final FunctionHandler functionHandler = new FunctionHandler(
        new IComplexElementListener() {
            public void elementLeaved() {
                FunctionInformation funcInfo = functionHandler.getFunctionInformation();
                if (funcInfo != null) {
                    funcsInfos.add(funcInfo);
                }
            }
        });

    /**
     * Constructs <code>FunctionsDocument</code> instance.
     *
     * @param elemLeavedListener listener to activate after parsing this element
     */
    FunctionsDocumentHandler(IComplexElementListener elemLeavedListener) {
        super("FunctionsDocument", elemLeavedListener);
    }

    /**
     * {@inheritDoc}
     */
    protected ElementHandler[] getSubHandlers() {
        return new ElementHandler[] {titleHandler, subTitleHandler, functionHandler};
    }

    /**
     * {@inheritDoc}
     */
    protected void enter(Attributes attrs) {
        title = null;
        subtitles.clear();
        funcsInfos.clear();
    }

    /**
     * Returns new {@link FunctionsDocument} instance corresponding to XML element processed by
     * this handler.
     *
     * @return information about processed "FunctionsDocument" element
     */
    public FunctionsDocument getFunctionsDocument() {
        if (title == null) {
            throw new FunctionsDocumentProcessingException(
                "Missing \"Title\" sub-element for " + getElementName() + " element.");
        }
        String[] subtitlesArray = (String[]) subtitles.toArray(new String[0]);
        FunctionInformation[] funcsInfosArray =
            (FunctionInformation[]) funcsInfos.toArray(new FunctionInformation[0]);
        return new FunctionsDocument(title, subtitlesArray, funcsInfosArray);
    }

}
