/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.funcinfoxml;

import java.io.IOException;
import java.io.InputStream;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import pl.chyla.wyjec.funcinfo.FunctionsDocument;
import pl.chyla.wyjec.utils.xml.XMLProcessor;


/**
 * Parser for "FunctionsDocument" XML documents that translates them to
 * {@link FunctionsDocument} objects.
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
public final class FunctionsDocumentParser {

    /**
     * Private empty constructor to disallow instantiating of the class.
     */
    private FunctionsDocumentParser() { }

    /**
     * Parses "FunctionsDocument" XML document from given input stream, generating object
     * representation of parsed data.
     *
     * @param input stream to read XML data from
     * @return new {@link FunctionsDocument} instance representing data from parsed XML document
     * @throws FunctionsDocumentProcessingException
     *     in case the XML document contains errors or it cannot be read from the stream
     */
    public static FunctionsDocument parseDocument(InputStream input)
        throws FunctionsDocumentProcessingException {
        FunctionsDocumentHandler topHandler = new FunctionsDocumentHandler(null);
        XMLProcessor processor = new XMLProcessor(new FunctionsDocumentDTDResolver());
        try {
            processor.processDocument(input, topHandler);
        } catch (ParserConfigurationException e) {
            throw new FunctionsDocumentProcessingException(e);
        } catch (SAXException e) {
            throw new FunctionsDocumentProcessingException(e);
        } catch (IOException e) {
            throw new FunctionsDocumentProcessingException(e);
        }
        return topHandler.getFunctionsDocument();
    }


    /**
     * Resolver for external entities, providing local copy of DTD for FunctionsDocument.
     *
     * @author Zbigniew Chyla (zbigniew@chyla.pl)
     */
    private static class FunctionsDocumentDTDResolver implements EntityResolver {

        /**
         * Default empty constructor.
         */
        FunctionsDocumentDTDResolver() { }

        /**
         * Resolves external entity by recognizing and handling only one system id that refers
         * to FunctionsDocument DTD.
         *
         * @param publicId the public identifier
         * @param systemId the system identifier or <code>null</code> if not available
         * @return
         *     object providing access to FunctionsDocument DTD if given system id is
         *     {@value FunctionsDocumentGenerator#FUNCTIONS_DOCUMENT_DTD_URI} and local copy
         *     of the DTD is available, <code>null</code> otherwise
         */
        public InputSource resolveEntity(String publicId, String systemId) {
            InputStream res;
            if (systemId != null
                && systemId.equals(FunctionsDocumentGenerator.FUNCTIONS_DOCUMENT_DTD_URI)) {
                res = getClass().getResourceAsStream("functions-document-1.0.dtd");
            } else {
                res = null;
            }
            if (res != null) {
                return new InputSource(res);
            } else {
                return null;
            }
        }

    }

}
