/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.funcinfoxml;

import org.xml.sax.Attributes;
import pl.chyla.wyjec.core.Type;
import pl.chyla.wyjec.funcinfo.ParameterInformation;
import pl.chyla.wyjec.reflection.TypeReflection;
import pl.chyla.wyjec.utils.xml.ComplexElementHandler;
import pl.chyla.wyjec.utils.xml.ContentElementHandler;
import pl.chyla.wyjec.utils.xml.ElementHandler;
import pl.chyla.wyjec.utils.xml.IComplexElementListener;
import pl.chyla.wyjec.utils.xml.IContentElementListener;


/**
 * Handler for "RequiredParameter" and "OptionalParameter" elements, providing new
 * {@link ParameterInformation} instance after XML parsing.
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
final class ParameterHandler extends ComplexElementHandler {

    /**
     * Value of "name" attribute
     */
    private String name;

    /**
     * Value of "type" attribute resolved to {@link Type} instance
     */
    private Type type;

    /**
     * Content of "Description" sub-element
     */
    private String description;

    /**
     * Handler used for processing "Description" sub-element
     */
    private final ContentElementHandler descriptionHandler = new ContentElementHandler(
        "Description", new IContentElementListener() {
            public void elementLeaved(String elemContent) {
                description = elemContent;
            }
        });

    /**
     * Constructs <code>ParameterHandler</code> instance supporting elements with given name.
     *
     * @param elemName name of supported XML element
     * @param elemLeavedListener listener to activate after parsing this element
     */
    ParameterHandler(String elemName, IComplexElementListener elemLeavedListener) {
        super(elemName, elemLeavedListener);
    }

    /**
     * {@inheritDoc}
     */
    protected ElementHandler[] getSubHandlers() {
        return new ElementHandler[] {descriptionHandler};
    }

    /**
     * {@inheritDoc}
     */
    protected void enter(Attributes attrs) {
        name = attrs.getValue("name");
        String typeName = attrs.getValue("type");
        if (typeName != null) {
            type = TypeReflection.getTypeByName(typeName);
        } else {
            type = null;
        }
        description = null;
    }

    /**
     * Returns new {@link ParameterInformation} instance corresponding to XML element processed
     * by this handler.
     *
     * @return information about processed "RequiredParameter" or "OptionalParameter" element
     * @throws FunctionsDocumentProcessingException
     *     in case the handler detected semantic errors during processing the XML document
     */
    ParameterInformation getParameterInformation() {
        if (name == null) {
            throw new FunctionsDocumentProcessingException(
                "Missing \"name\" attribute for " + getElementName() + " element.");
        }
        if (type == null) {
            throw new FunctionsDocumentProcessingException(
                "Missing \"type\" attribute for " + getElementName() + " element.");
        }
        return new ParameterInformation(name, type, description);
    }

}
