/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.reflection;

import java.util.Map;
import java.util.TreeMap;
import pl.chyla.wyjec.core.OptParameters;
import pl.chyla.wyjec.core.Parameter;
import pl.chyla.wyjec.core.Type;
import pl.chyla.wyjec.core.optparameters.BooleanOptParameters;
import pl.chyla.wyjec.core.optparameters.DateOptParameters;
import pl.chyla.wyjec.core.optparameters.FloatOptParameters;
import pl.chyla.wyjec.core.optparameters.IntegerOptParameters;
import pl.chyla.wyjec.core.optparameters.StringOptParameters;
import pl.chyla.wyjec.core.parameters.BooleanParameter;
import pl.chyla.wyjec.core.parameters.DateParameter;
import pl.chyla.wyjec.core.parameters.FloatParameter;
import pl.chyla.wyjec.core.parameters.IntegerParameter;
import pl.chyla.wyjec.core.parameters.StringParameter;


/**
 * Helper class offering operations on objects related to
 * {@link pl.chyla.wyjec.core.FunctionSignature}: {@link Parameter} and {@link OptParameters}
 * which are not available directly from their classes because they are "unsafe", i.e. they
 * don't guarantee the appropriate static checks. These operations are not needed during normal
 * Wyjec operation, but they may be useful in some types of Wyjec tools.
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
public final class FunctionSignatureReflection {

    /**
     * Mapping from fully qualified names of direct {@link Parameter} subclasses to types of
     * parameters represented by them.
     */
    private static final Map PARAM_TYPES_BY_CLASS_NAMES;
    static {
        PARAM_TYPES_BY_CLASS_NAMES = new TreeMap();
        PARAM_TYPES_BY_CLASS_NAMES.put(BooleanParameter.class.getName(), Type.BOOLEAN);
        PARAM_TYPES_BY_CLASS_NAMES.put(IntegerParameter.class.getName(), Type.INTEGER);
        PARAM_TYPES_BY_CLASS_NAMES.put(StringParameter.class.getName(), Type.STRING);
        PARAM_TYPES_BY_CLASS_NAMES.put(FloatParameter.class.getName(), Type.FLOAT);
        PARAM_TYPES_BY_CLASS_NAMES.put(DateParameter.class.getName(), Type.DATE);
    }

    /**
     * Mapping from fully qualified names of direct {@link OptParameters} subclasses to types of
     * parameters represented by them.
     */
    private static final Map OPT_PARAM_TYPES_BY_CLASS_NAMES;
    static {
        OPT_PARAM_TYPES_BY_CLASS_NAMES = new TreeMap();
        OPT_PARAM_TYPES_BY_CLASS_NAMES.put(BooleanOptParameters.class.getName(), Type.BOOLEAN);
        OPT_PARAM_TYPES_BY_CLASS_NAMES.put(IntegerOptParameters.class.getName(), Type.INTEGER);
        OPT_PARAM_TYPES_BY_CLASS_NAMES.put(StringOptParameters.class.getName(), Type.STRING);
        OPT_PARAM_TYPES_BY_CLASS_NAMES.put(FloatOptParameters.class.getName(), Type.FLOAT);
        OPT_PARAM_TYPES_BY_CLASS_NAMES.put(DateOptParameters.class.getName(), Type.DATE);
    }

    /**
     * Private empty constructor to disallow instantiating of the class.
     */
    private FunctionSignatureReflection() { }

    /**
     * Returns a type of parameter represented by direct {@link Parameter} subclass specified by
     * its fully qualified name.
     *
     * @param clsName fully qualified name of parameter class
     * @return
     *     type of parameter represented by specified class or <code>null</code> if given name
     *     doesn't specify any direct subclass of {@link Parameter}
     */
    public static Type getParameterTypeByClassName(String clsName) {
        return (Type) PARAM_TYPES_BY_CLASS_NAMES.get(clsName);
    }

    /**
     * Returns a type of parameters in optional parameters group represented by direct
     * {@link OptParameters} subclass specified by its fully qualified name.
     *
     * @param clsName fully qualified name of optional parameters group class
     * @return
     *     type of parameters in group represented by specified class or <code>null</code> if
     *     given name doesn't specify any direct subclass of {@link OptParameters}
     */
    public static Type getOptParametersTypeByClassName(String clsName) {
        return (Type) OPT_PARAM_TYPES_BY_CLASS_NAMES.get(clsName);
    }

    /**
     * Returns new instance of one of direct {@link Parameter} subclasses corresponding to given
     * parameter's type.
     *
     * @param paramType type of the parameter
     * @return
     *     new instance of one of {@link Parameter} subclasses such that
     *     {@link Parameter#getType()} returns specified type
     * @throws IllegalArgumentException
     *     in case given type is not "real" Wyjec type (other than {@link Type#NONE})
     */
    public static Parameter newParameterByType(Type paramType) {
        Parameter param;
        switch (paramType.getEncodedType()) {
        case Type.BOOLEAN_ENCODED:
            param = new BooleanParameter();
            break;
        case Type.INTEGER_ENCODED:
            param = new IntegerParameter();
            break;
        case Type.STRING_ENCODED:
            param = new StringParameter();
            break;
        case Type.FLOAT_ENCODED:
            param = new FloatParameter();
            break;
        case Type.DATE_ENCODED:
            param = new DateParameter();
            break;
        default:
            throw new IllegalArgumentException(
                "Invalid parameter type given: " + paramType.getName());
        }
        return param;
    }

    /**
     * Returns new instance of one of direct {@link OptParameters} subclasses corresponding to
     * given parameter's type.
     *
     * @param paramsType type of each optional parameter in the optional parameters group
     * @return
     *     new instance of one of {@link OptParameters} subclasses such that
     *     {@link OptParameters#getType()} returns specified type
     * @throws IllegalArgumentException
     *     in case given type is not "real" Wyjec type (other than {@link Type#NONE})
     */
    public static OptParameters newOptParametersByType(Type paramsType) {
        OptParameters optParams;
        switch (paramsType.getEncodedType()) {
        case Type.BOOLEAN_ENCODED:
            optParams = new BooleanOptParameters();
            break;
        case Type.INTEGER_ENCODED:
            optParams = new IntegerOptParameters();
            break;
        case Type.STRING_ENCODED:
            optParams = new StringOptParameters();
            break;
        case Type.FLOAT_ENCODED:
            optParams = new FloatOptParameters();
            break;
        case Type.DATE_ENCODED:
            optParams = new DateOptParameters();
            break;
        default:
            throw new IllegalArgumentException(
                "Invalid parameters type given: " + paramsType.getName());
        }
        return optParams;
    }

}
