/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.reflection;

import java.util.Map;
import java.util.TreeMap;
import pl.chyla.wyjec.core.Function;
import pl.chyla.wyjec.core.OptParameters;
import pl.chyla.wyjec.core.Parameter;
import pl.chyla.wyjec.core.Type;
import pl.chyla.wyjec.core.Value;
import pl.chyla.wyjec.core.functions.BooleanFunction;
import pl.chyla.wyjec.core.functions.DateFunction;
import pl.chyla.wyjec.core.functions.FloatFunction;
import pl.chyla.wyjec.core.functions.IntegerFunction;
import pl.chyla.wyjec.core.functions.StringFunction;
import pl.chyla.wyjec.core.optparameters.BooleanOptParameters;
import pl.chyla.wyjec.core.optparameters.DateOptParameters;
import pl.chyla.wyjec.core.optparameters.FloatOptParameters;
import pl.chyla.wyjec.core.optparameters.IntegerOptParameters;
import pl.chyla.wyjec.core.optparameters.StringOptParameters;
import pl.chyla.wyjec.core.parameters.BooleanParameter;
import pl.chyla.wyjec.core.parameters.DateParameter;
import pl.chyla.wyjec.core.parameters.FloatParameter;
import pl.chyla.wyjec.core.parameters.IntegerParameter;
import pl.chyla.wyjec.core.parameters.StringParameter;
import pl.chyla.wyjec.utils.ReadableRawString;


/**
 * Abstract implementation of {@link Function} which allows you to implement functions in an
 * unsafe way, i.e. without standard static checks. Subclasses of this class can access the
 * actual parameters by their indices, using {@link Value} objects directly. Also function's
 * result is returned in raw {@link Value} object from {@link Function#typedEvaluate(Value)}.
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
public abstract class ReflectiveFunction extends Function {

    /**
     * Mapping from fully qualified names of base function classes to their return types.
     */
    private static final Map RET_TYPES_BY_CLASS_NAMES;
    static {
        RET_TYPES_BY_CLASS_NAMES = new TreeMap();
        RET_TYPES_BY_CLASS_NAMES.put(BooleanFunction.class.getName(), Type.BOOLEAN);
        RET_TYPES_BY_CLASS_NAMES.put(IntegerFunction.class.getName(), Type.INTEGER);
        RET_TYPES_BY_CLASS_NAMES.put(StringFunction.class.getName(), Type.STRING);
        RET_TYPES_BY_CLASS_NAMES.put(FloatFunction.class.getName(), Type.FLOAT);
        RET_TYPES_BY_CLASS_NAMES.put(DateFunction.class.getName(), Type.DATE);
    }

    /**
     * Gets the return type of a function specified by fully qualified name of its base class,
     * which must be one of standard direct subclasses of {@link Function}.
     *
     * @param funcClsName fully qualified name of base class of the function
     * @return
     *     return type of the function or <code>null</code> if given name doesn't specify any
     *     direct subclass of {@link Function}
     */
    public static Type getResultTypeByFunctionClassName(String funcClsName) {
        return (Type) RET_TYPES_BY_CLASS_NAMES.get(funcClsName);
    }

    /**
     * Returns value of the actual parameter with given index.
     *
     * @param index 0-based index of the parameter
     * @return new {@link Value} instance containing the parameter's value
     * @throws ArrayIndexOutOfBoundsException
     *     in case given index is negative or it's not less than the number of actual parameters
     */
    protected final Value getParameterByIndex(int index) {
        int reqParamsCount = getSignature().getRequiredParametersTypes().length;
        if (index < reqParamsCount) {
            return getRequiredParameterByIndex(index);
        } else {
            return getOptionalParameterByIndex(index - reqParamsCount);
        }
    }

    /**
     * Returns value of actual parameter corresponding to declared required parameter with given
     * index.
     *
     * @param index 0-based index of the parameter
     * @return new {@link Value} instance containing the parameter's value
     * @throws ArrayIndexOutOfBoundsException
     *     in case given index is negative or it's not less than the number of declared required
     *     parameters of this function
     */
    private Value getRequiredParameterByIndex(int index) {
        Parameter[] reqParams = getRequiredParameters();
        Type reqParamType = getSignature().getRequiredParametersTypes()[index];
        Value result;
        switch (reqParamType.getEncodedType()) {
        case Type.BOOLEAN_ENCODED:
            boolean booleanValue = getParameter((BooleanParameter) reqParams[index]);
            result = new Value(0);
            result.setBoolean(booleanValue);
            break;
        case Type.INTEGER_ENCODED:
            long integerValue = getParameter((IntegerParameter) reqParams[index]);
            result = new Value(0);
            result.setInteger(integerValue);
            break;
        case Type.STRING_ENCODED:
            ReadableRawString stringValue = getParameter((StringParameter) reqParams[index]);
            result = new Value(stringValue.getLength());
            result.setString(stringValue);
            break;
        case Type.FLOAT_ENCODED:
            result = new Value(0);
            result.setFloat(getParameter((FloatParameter) reqParams[index]));
            break;
        case Type.DATE_ENCODED:
            result = new Value(0);
            result.setDate(getParameter((DateParameter) reqParams[index]));
            break;
        default:
            throw new RuntimeException(
                "Invalid type of required parameter: " + reqParamType.toString());
        }
        return result;
    }

    /**
     * Returns value of actual parameter corresponding to declared optional parameter with given
     * index, relative to index of the first optional parameter.
     *
     * @param optIndex
     *     0-based index of the parameter, relative to index of the first optional parameter
     * @return new {@link Value} instance containing the parameter's value
     * @throws ArrayIndexOutOfBoundsException
     *     in case given index is negative or it's not less than the number of actual parameters
     *     beyond the required ones
     */
    private Value getOptionalParameterByIndex(int optIndex) {
        OptParameters optParams = getOptionalParameters();
        if (optParams == null) {
            throw new ArrayIndexOutOfBoundsException(optIndex);
        }
        Type optParamType = getSignature().getOptionalParametersType();
        Value result;
        switch (optParamType.getEncodedType()) {
        case Type.BOOLEAN_ENCODED:
            result = new Value(0);
            result.setBoolean(getParameter((BooleanOptParameters) optParams, optIndex));
            break;
        case Type.INTEGER_ENCODED:
            result = new Value(0);
            result.setInteger(getParameter((IntegerOptParameters) optParams, optIndex));
            break;
        case Type.STRING_ENCODED:
            ReadableRawString stringValue = getParameter(
                (StringOptParameters) optParams, optIndex);
            result = new Value(stringValue.getLength());
            result.setString(stringValue);
            break;
        case Type.FLOAT_ENCODED:
            result = new Value(0);
            result.setFloat(getParameter((FloatOptParameters) optParams, optIndex));
            break;
        case Type.DATE_ENCODED:
            result = new Value(0);
            result.setDate(getParameter((DateOptParameters) optParams, optIndex));
            break;
        default:
            throw new RuntimeException(
                "Invalid type of optional parameter: " + optParamType.toString());
        }
        return result;
    }

    /**
     * Returns an array of objects representing function's declared parameters in the
     * appropriate order. These must be the same objects that were used in constructor's
     * parameters when creating {@link pl.chyla.wyjec.core.FunctionSignature} instance returned
     * by {@link Function#getSignature()}.
     *
     * @return
     *     array of {@link Parameter} instances corresponding to this function's required
     *     parameters
     */
    protected abstract Parameter[] getRequiredParameters();

    /**
     * Returns object representing a list of optional function's parameters. This must be the
     * same object that was used as constructor's parameters when creating
     * {@link pl.chyla.wyjec.core.FunctionSignature} instance returned by
     * {@link Function#getSignature()}.
     *
     * @return
     *     {@link OptParameters} instance corresponding to this function's optional parameters
     *     or <code>null</code> if this function doesn't accept optional parameters
     */
    protected abstract OptParameters getOptionalParameters();

}
