/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.reflection;

import java.util.Map;
import java.util.TreeMap;
import pl.chyla.wyjec.core.Type;


/**
 * Helper class offering operations on {@link Type} objects which are not available directly
 * from their class because they are "unsafe", i.e. they don't guarantee the appropriate static
 * checks. These operations are not needed during normal Wyjec operation, but they may be useful
 * in some types of Wyjec tools.
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
public final class TypeReflection {

    /**
     * Array containing references to all known {@link Type} instances.
     */
    private static final Type[] ALL_TYPES = {
        Type.NONE, Type.BOOLEAN, Type.INTEGER, Type.STRING, Type.FLOAT, Type.DATE};

    /**
     * Mapping from types' names (as returned by {@link Type#getName()}) to {@link Type}
     * instances.
     */
    private static final Map TYPES_BY_NAMES;
    static {
        TYPES_BY_NAMES = new TreeMap();
        for (int i = 0; i < ALL_TYPES.length; i++) {
            Type type = ALL_TYPES[i];
            TYPES_BY_NAMES.put(type.getName(), type);
        }
    }

    /**
     * Private empty constructor to disallow instantiating of the class.
     */
    private TypeReflection() { }

    /**
     * Returns {@link Type} instance for a type specified by its name.
     *
     * @param name name of a type, as returned by {@link Type#getName()}
     * @return
     *     {@link Type} instance corresponding to given name or <code>null</code> if there's no
     *     type with given name
     */
    public static Type getTypeByName(String name) {
        return (Type) TYPES_BY_NAMES.get(name);
    }

}
