/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.simulation;

import pl.chyla.wyjec.core.FunctionSignature;
import pl.chyla.wyjec.core.OptParameters;
import pl.chyla.wyjec.core.Parameter;
import pl.chyla.wyjec.core.Type;
import pl.chyla.wyjec.core.Value;
import pl.chyla.wyjec.funcinfo.FunctionInformation;
import pl.chyla.wyjec.funcinfo.ParameterInformation;
import pl.chyla.wyjec.reflection.FunctionSignatureReflection;
import pl.chyla.wyjec.reflection.ReflectiveFunction;


/**
 * Implementation of Wyjec function designed for use in simulations when the real implementation
 * of some functions is not available. This class gets name and signature of the function from
 * its description contained in {@link FunctionInformation} instance. To evaluate value of the
 * function a separate object is used ({@link IFunctionHandler}).
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
final class SimulatedFunction extends ReflectiveFunction {

    /**
     * Description of the simulated function.
     */
    private final FunctionInformation funcInfo;

    /**
     * Handler providing evaluation results of this function.
     */
    private final IFunctionHandler funcHandler;

    /**
     * This function's signature.
     */
    private final FunctionSignature signature;

    /**
     * Declared required parameters of this function.
     */
    private final Parameter[] reqParams;

    /**
     * Group of optional parameters of this function (or <code>null</code> if optional
     * parameters are not supported).
     */
    private final OptParameters optParams;

    /**
     * Constructs <code>SimulatedFunction</code> instance for function described by given
     * {@link FunctionInformation} object, using given handler for obtaining this function's
     * evaluation results.
     *
     * @param funcInfo description of a function to simulate
     * @param funcHandler handler responsible for providing this function's evaluation results
     */
    SimulatedFunction(FunctionInformation funcInfo, IFunctionHandler funcHandler) {
        this.funcInfo = funcInfo;
        this.funcHandler = funcHandler;
        ParameterInformation[] reqParamsDescs = funcInfo.getRequiredParameters();
        reqParams = new Parameter[reqParamsDescs.length];
        for (int i = 0; i < reqParamsDescs.length; i++) {
            Type paramType = reqParamsDescs[i].getType();
            reqParams[i] = FunctionSignatureReflection.newParameterByType(paramType);
        }
        ParameterInformation optParamsDesc = funcInfo.getOptionalParameters();
        if (optParamsDesc != null) {
            Type optParamsType = optParamsDesc.getType();
            optParams = FunctionSignatureReflection.newOptParametersByType(optParamsType);
        } else {
            optParams = null;
        }
        this.signature = new FunctionSignature(reqParams, optParams);
    }

    /**
     * {@inheritDoc}
     *
     * @param result {@inheritDoc}
     */
    protected void typedEvaluate(Value result) {
        int actParamsCount = reqParams.length;
        if (optParams != null) {
            actParamsCount += getOptParametersCount(optParams);
        }
        Value[] actParams = new Value[actParamsCount];
        for (int i = 0; i < actParamsCount; i++) {
            actParams[i] = getParameterByIndex(i);
        }
        funcHandler.evaluateFunction(funcInfo, result, actParams);
    }

    /**
     * {@inheritDoc}
     *
     * @return {@inheritDoc}
     */
    public Type getResultType() {
        return funcInfo.getResultType();
    }

    /**
     * {@inheritDoc}
     *
     * @return {@inheritDoc}
     */
    public FunctionSignature getSignature() {
        return signature;
    }

    /**
     * {@inheritDoc}
     *
     * @return {@inheritDoc}
     */
    public String getName() {
        return funcInfo.getName();
    }

    /**
     * {@inheritDoc}
     *
     * @return {@inheritDoc}
     */
    protected Parameter[] getRequiredParameters() {
        return reqParams;
    }

    /**
     * {@inheritDoc}
     *
     * @return {@inheritDoc}
     */
    protected OptParameters getOptionalParameters() {
        return optParams;
    }

}
