/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.simulation;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.LinkedList;
import java.util.List;
import pl.chyla.wyjec.core.Function;
import pl.chyla.wyjec.funcinfo.FunctionInformation;
import pl.chyla.wyjec.funcinfo.FunctionsDocument;
import pl.chyla.wyjec.funcinfoxml.FunctionsDocumentParser;
import pl.chyla.wyjec.funcinfoxml.FunctionsDocumentProcessingException;


/**
 * Simple factory that generates simulated Wyjec functions by using their descriptions contained
 * in specified XML documents. All generated functions will use specified
 * {@link IFunctionHandler} instance to simulate evaluation.
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
public final class SimulatedFunctionsFactory {

    /**
     * Private empty constructor to disallow instantiating of the class.
     */
    private SimulatedFunctionsFactory() { }

    /**
     * Generates an array of {@link Function} instances that simulate all functions described in
     * specified XML documents.
     *
     * @param funcsDocsPaths
     *     an array of paths to XML files containing descriptions of Wyjec functions
     * @param funcHandler
     *     object used for obtaining simulated evaluation results for all generated functions
     * @return a list of simulated functions that use specified handler for obtaining results
     * @throws IOException in case some of specified files can't be accessed
     * @throws FunctionsDocumentProcessingException
     *     in case of errors processing some of the XML documents
     */
    public static Function[] generateFunctionsFromDocs(
        String[] funcsDocsPaths, IFunctionHandler funcHandler)
        throws IOException, FunctionsDocumentProcessingException {
        List funcs = new LinkedList();
        for (int i = 0; i < funcsDocsPaths.length; i++) {
            InputStream input = new BufferedInputStream(new FileInputStream(funcsDocsPaths[i]));
            FunctionsDocument funcsDoc = FunctionsDocumentParser.parseDocument(input);
            FunctionInformation[] funcsInfos = funcsDoc.getFunctionsInformations();
            for (int j = 0; j < funcsInfos.length; j++) {
                funcs.add(new SimulatedFunction(funcsInfos[j], funcHandler));
            }
        }
        return (Function[]) funcs.toArray(new Function[0]);
    }

}
