/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.utils.cli;

import java.util.List;
import java.util.Map;
import java.util.Set;


/**
 * Represents the programs command line in processed form, after separating options and their
 * arguments from the actual arguments of the program.
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
public final class CommandLine {

    /**
     * A collection of argument-less options specified in the command line.
     */
    private final Set noArgOptions;

    /**
     * Arguments for options specified in the command line. For each used option there's a list
     * of arguments, each element corresponds to single occurrence of the option in the command
     * line.
     */
    private final Map argsListsForArgOptions;

    /**
     * List of the actual arguments, not recognized as options or options' arguments.
     */
    private final List argsList;

    /**
     * Constructs <code>CommandLine</code> instance containing specified information from the
     * command line.
     *
     * @param noArgOptions argument-less options specified in the command line
     * @param argsListsForArgOptions
     *     mapping from strings to lists of strings, where a key is the name of option given in
     *     the command line with an argument one or more times and mapped value is a list of
     *     option's arguments (one for each occurrence, in the same order)
     * @param argsList
     *     the actual arguments from the command line, not recognized as options or options'
     *     arguments
     */
    CommandLine(Set noArgOptions, Map argsListsForArgOptions, List argsList) {
        this.noArgOptions = noArgOptions;
        this.argsListsForArgOptions = argsListsForArgOptions;
        this.argsList = argsList;
    }

    /**
     * Informs whether specified option has occurred on the command line (with or without an
     * argument).
     *
     * @param optName name of the option
     * @return <code>true</code> if the option has been used <code>false</code> otherwise
     */
    public boolean isOptionSet(String optName) {
        return noArgOptions.contains(optName) || argsListsForArgOptions.containsKey(optName);
    }

    /**
     * Returns an array of arguments given in the command line for specified option. The array
     * will contain the option's arguments for all its occurrences, in the same order.
     *
     * @param optName name of the option
     * @return
     *     newly created array of arguments specified for all occurrences of the option in the
     *     command line
     */
    public String[] getOptionArguments(String optName) {
        List optArgsList = (List) argsListsForArgOptions.get(optName);
        if (optArgsList != null) {
            return (String[]) optArgsList.toArray(new String[0]);
        } else {
            return new String[0];
        }
    }

    /**
     * Returns the actual arguments from the command line, i.e. elements not recognized as
     * options or options' arguments.
     *
     * @return newly created array of arguments
     */
    public String[] getArguments() {
        return (String[]) argsList.toArray(new String[0]);
    }

}
