/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.utils.cli;

import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;


/**
 * Simple parser for command line options, separating options from the real arguments and
 * storing the result in {@link CommandLine} instance.
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
public final class CommandLineParser {

    /**
     * Collection of supported options that don't require arguments.
     */
    private final Set noArgOptions;

    /**
     * Collection of supported options that require argument.
     */
    private final Set argOptions;

    /**
     * Constructs <code>CommandLineParser</code> instance that supports specified command-line
     * options.
     *
     * @param optsWithoutArgs
     *     array of options that don't require arguments, each option must be specified using
     *     its full form, e.g. "-v", "-quiet"
     * @param optsWithArgs
     *     array of options that requite argument, each option must be specified using its full
     *     form, e.g. "-n", "-repetitions"
     */
    public CommandLineParser(String[] optsWithoutArgs, String[] optsWithArgs) {
        noArgOptions = new HashSet();
        for (int i = 0; i < optsWithoutArgs.length; i++) {
            noArgOptions.add(optsWithoutArgs[i]);
        }
        argOptions = new HashSet();
        for (int i = 0; i < optsWithArgs.length; i++) {
            argOptions.add(optsWithArgs[i]);
        }
    }

    /**
     * Parses given array of input arguments (as passed in to "main" method) and produces the
     * corresponding instance of {@link CommandLine}.
     *
     * @param inputArgs array of arguments from the program's command line
     * @throws CommandLineSyntaxException
     *     in case the parser detects an error in given array of arguments: missing argument for
     *     option that requires it or option without arguments used more than once
     * @return
     *     {@link CommandLine} instance containing information about processed options and
     *     filtered out arguments of the program
     */
    public CommandLine parse(String[] inputArgs) throws CommandLineSyntaxException {
        List leftArgsList = new LinkedList();
        Set usedNoArgOptions = new HashSet();
        Map usedArgsForArgOptions = new HashMap();
        int idx = 0;
        while (idx < inputArgs.length) {
            String inputArg = inputArgs[idx];
            if (noArgOptions.contains(inputArg)) {
                if (!usedNoArgOptions.contains(inputArg)) {
                    usedNoArgOptions.add(inputArg);
                    idx++;
                } else {
                    throw new CommandLineSyntaxException("Duplicate option: " + inputArg);
                }
            } else if (argOptions.contains(inputArg)) {
                idx++;
                if (idx < inputArgs.length) {
                    List optArgs = (List) usedArgsForArgOptions.get(inputArg);
                    if (optArgs == null) {
                        optArgs = new LinkedList();
                        usedArgsForArgOptions.put(inputArg, optArgs);
                    }
                    optArgs.add(inputArgs[idx]);
                    idx++;
                } else {
                    throw new CommandLineSyntaxException(
                        "Missing argument for option: " + inputArg);
                }
            } else {
                leftArgsList.add(inputArg);
                idx++;
            }
        }
        return new CommandLine(usedNoArgOptions, usedArgsForArgOptions, leftArgsList);
    }


    /**
     * Exception indicating syntax error in a sequence of program's arguments.
     *
     * @author Zbigniew Chyla (zbigniew@chyla.pl)
     */
    public static class CommandLineSyntaxException extends Exception {

        /**
         * Constructs <code>CommandLineSyntaxException</code> instance using specified error
         * message.
         *
         * @param msg the detailed error message
         */
        public CommandLineSyntaxException(String msg) {
            super(msg);
        }

    }

}
