/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.utils.xml;

import java.util.Map;
import java.util.TreeMap;


/**
 * Base class for element handlers which have some sub-handlers, responsible for processing
 * XML elements inside the one supported by a handler.
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
public abstract class ComplexElementHandler extends ElementHandler {

    /**
     * Listener to be notified after reaching the end of supported element.
     */
    private final IComplexElementListener elemLeavedListener;

    /**
     * Collection of sub-handlers accessible by their names. This field is constructed lazily,
     * using the information got from {@link #getSubHandlers()} method. Before accessing this
     * field, one should call {@link #assureSubHandlersByElements()} method.
     */
    private Map subHandlersByElements;

    /**
     * Constructs <code>ComplexElementHandler</code> for specified XML element using given
     * listener.
     *
     * @param elemName name of supported element
     * @param elemListener
     *     listener to notify about encountering the end of the XML element during processing
     */
    protected ComplexElementHandler(String elemName, IComplexElementListener elemListener) {
        super(elemName);
        this.elemLeavedListener = elemListener;
    }

    /**
     * Method called after encountering the end of supported XML element during processing,
     * forwards this notification to the associated listener, providing the element's content.
     *
     * @param elemContent {@inheritDoc}
     */
    protected final void leave(String elemContent) {
        if (elemLeavedListener != null) {
            elemLeavedListener.elementLeaved();
        }
    }

    /**
     * {@inheritDoc}
     *
     * @return {@inheritDoc}
     */
    final boolean hasSubHandlers() {
        assureSubHandlersByElements();
        return subHandlersByElements.size() != 0;
    }

    /**
     * {@inheritDoc}
     *
     * @param subElemName {@inheritDoc}
     * @return {@inheritDoc}
     */
    final ElementHandler getHandlerForSubElement(String subElemName) {
        assureSubHandlersByElements();
        return (ElementHandler) subHandlersByElements.get(subElemName);
    }

    /**
     * Creates a map of sub-handler by their names and assigns it to
     * {@link #subHandlersByElements} field, unless the field is already not-null.
     */
    private void assureSubHandlersByElements() {
        if (subHandlersByElements == null) {
            subHandlersByElements = new TreeMap();
            ElementHandler[] subHandlers = getSubHandlers();
            for (int i = 0; i < subHandlers.length; i++) {
                ElementHandler subHandler = subHandlers[i];
                subHandlersByElements.put(subHandler.getElementName(), subHandler);
            }
        }
    }

    /**
     * Returns an array of sub-handlers of this element handler.
     *
     * @return this handler's sub-handlers
     */
    protected abstract ElementHandler[] getSubHandlers();

}
