/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.utils.xml;

import org.xml.sax.Attributes;


/**
 * Element handler for XML elements that contain character data. This handler has no
 * sub-handlers and provides only its element's text content for further processing.
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
public final class ContentElementHandler extends ElementHandler {

    /**
     * Listener to be notified after reaching the end of supported element and collecting
     * contained character data.
     */
    private final IContentElementListener elemListener;

    /**
     * Constructs <code>ContentElementHandler</code> for specified XML element using given
     * listener.
     *
     * @param elemName name of supported element
     * @param elemListener
     *     listener to notify about encountering the end of the XML element during processing
     */
    public ContentElementHandler(String elemName, IContentElementListener elemListener) {
        super(elemName);
        this.elemListener = elemListener;
    }

    /**
     * {@inheritDoc}
     *
     * @param attrs {@inheritDoc}
     */
    protected void enter(Attributes attrs) {
        // nothing to do
    }

    /**
     * Method called after encountering the end of supported XML element during processing,
     * forwards this notification to the associated listener, providing the element's content.
     *
     * @param elemContent {@inheritDoc}
     */
    protected void leave(String elemContent) {
        if (elemListener != null) {
            elemListener.elementLeaved(elemContent);
        }
    }

    /**
     * Always returns <code>false</code> since this element handler has no sub-handlers.
     *
     * @return <code>false</code>
     */
    boolean hasSubHandlers() {
        return false;
    }

    /**
     * Always returns <code>null</code> since this element handler has no sub-handlers.
     *
     * @param subElemName {@inheritDoc}
     * @return <code>null</code>
     */
    ElementHandler getHandlerForSubElement(String subElemName) {
        return null;
    }

}
