/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.utils.xml;

import org.xml.sax.Attributes;


/**
 * Base abstract class for element handlers, i.e. classes responsible for processing selected
 * XML elements. Each handler supports single XML element (which may occur many times in XML
 * document) and optionally contains references to sub-handlers, that support XML elements
 * contained in the one supported by the handler.
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
public abstract class ElementHandler {

    /**
     * Name of XML element supported by this handler.
     */
    private final String elemName;

    /**
     * Constructs <code>ElementHandler</code> for specified XML element.
     *
     * @param elemName name of supported element
     */
    protected ElementHandler(String elemName) {
        this.elemName = elemName;
    }

    /**
     * Returns the name of the XML element supported by this handler.
     *
     * @return supported element's name
     */
    protected final String getElementName() {
        return elemName;
    }

    /**
     * Method called after encountering the start of XML element supported by this handler
     * during processing.
     *
     * @param attrs attributes of the element
     */
    protected abstract void enter(Attributes attrs);

    /**
     * Method called after encountering the end of XML element supported by this handler during
     * processing.
     *
     * @param elemContent
     *     character data contained in the element or </code>null</code> if the content is not
     *     available; the content will be available if and only if the handler has no
     *     sub-handlers ({@link #hasSubHandlers()} returns <code>false</code>)
     */
    protected abstract void leave(String elemContent);

    /**
     * Checks if this handler has any sub-handlers, i.e. handlers which support XML elements
     * contained in the one supported by this handler. For any given instance this method should
     * always return the same value.
     *
     * @return
     *     <code>true</code> if this handler has some sub-handlers, <code>false</code> otherwise
     */
    abstract boolean hasSubHandlers();

    /**
     * Returns sub-handler of this handler that supports specified XML element.
     *
     * @param subElemName name of XML element to be supported by sub-handler
     * @return
     *     sub-handler supporting specified element or <code>null</code> if no such sub-handler
     *     exists
     */
    abstract ElementHandler getHandlerForSubElement(String subElemName);

}
