/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.utils.xml;

import java.util.Stack;
import pl.chyla.wyjec.utils.IntStack;


/**
 * Simple implementation of stack for which each stored element contains a reference to element
 * handler and level number of XML element for which the handler is used.
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
final class ElementHandlersStack {

    /**
     * Stack containing {@link ElementHandler} instances.
     */
    private final Stack handlers;

    /**
     * Stack containing levels numbers associated with the appropriate elements on
     * {@link #handlers} stack.
     */
    private final IntStack levels;

    /**
     * Constructs <code>HandlersStack</code> instance with no elements on the stack.
     */
    ElementHandlersStack() {
        levels = new IntStack();
        handlers = new Stack();
    }

    /**
     * Pushes given element handler together with associated level number at the top of this
     * stack.
     *
     * @param handler element handler to push on this stack
     * @param level level number associated with the handler
     */
    void push(ElementHandler handler, int level) {
        levels.push(level);
        handlers.push(handler);
    }

    /**
     * Pops element handler from the top of this stack.
     */
    void pop() {
        levels.pop();
        handlers.pop();
    }

    /**
     * Returns element handler that is currently at the top of this stack without changing the
     * stack's state.
     *
     * @return element handler at the top of this stack
     */
    ElementHandler peekTopHandler() {
        return (ElementHandler) handlers.peek();
    }

    /**
     * Returns level number associated with element handler that is currently at the top of this
     * stack without changing the stack's state.
     *
     * @return level number associated with element handler at the top of this stack
     */
    int peekTopLevel() {
        return levels.peek();
    }

}
