/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.utils.xml;

import java.util.regex.Matcher;
import java.util.regex.Pattern;


/**
 * Utility class for decoding character and entity references nested in XML fragments. It
 * supports all character references and predefined XML entities (amp, lt, gt, apos, quot).
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
public final class StandardXMLReferencesDecoder {

    /**
     * Regular expression that matches character reference or predefined entity reference.
     * It contains a single group that refers to characters between "&" and ";" (entity name in
     * case of entity reference).
     */
    private static final Pattern REFERENCE_PATTERN = Pattern.compile(
        "&(#[0-9]+|#x[0-9a-fA-F]+|amp|lt|gt|apos|quot);");

    /**
     * Private empty constructor to disallow instantiating of the class.
     */
    private StandardXMLReferencesDecoder() { }

    /**
     * Decodes all character and entity references contained in given XML fragment and returns
     * new string with its decoded equivalent.
     *
     * @param inputText input text with character and entity references to decode
     * @return decoded text
     */
    public static String decode(String inputText) {
        Matcher matcher = REFERENCE_PATTERN.matcher(inputText);
        StringBuffer result = new StringBuffer();
        int inputPos = 0;
        while (matcher.find(inputPos)) {
            if (matcher.start() > inputPos) {
                result.append(inputText.substring(inputPos, matcher.start()));
            }
            String name = matcher.group(1);
            if (name.equals("amp")) {
                result.append('&');
            } else if (name.equals("lt")) {
                result.append('<');
            } else if (name.equals("gt")) {
                result.append('>');
            } else if (name.equals("apos")) {
                result.append('\'');
            } else if (name.equals("quot")) {
                result.append('"');
            } else if (name.startsWith("#x")) {
                int value = Integer.parseInt(name.substring(2), 0x10);
                if (value >= Character.MIN_VALUE && value <= Character.MAX_VALUE) {
                    result.append((char) value);
                } else {
                    result.append(matcher.group());
                }
            } else if (name.startsWith("#")) {
                int value = Integer.parseInt(name.substring(1));
                if (value >= Character.MIN_VALUE && value <= Character.MAX_VALUE) {
                    result.append((char) value);
                } else {
                    result.append(matcher.group());
                }
            }
            inputPos = matcher.end();
        }
        result.append(inputText.substring(inputPos));
        return result.toString();
    }

}
