/*
 * Tools for Wyjec - expression language, version 1.0
 * (c) Copyright by Zbigniew Chyla 2005
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if
 * not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
package pl.chyla.wyjec.utils.xml;

import java.io.IOException;
import java.io.InputStream;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import org.xml.sax.Attributes;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;


/**
 * Helper class used for parsing XML document and translating it to a domain-specific
 * representation. This class contains only very generic processing engine, creating domain
 * objects is delegated to domain-specific handlers of XML elements ({@link ElementHandler}).
 *
 * @author Zbigniew Chyla (zbigniew@chyla.pl)
 */
public final class XMLProcessor {

    /**
     * Factory for SAX parsers used by this class.
     */
    private static final SAXParserFactory parserFactory = SAXParserFactory.newInstance();

    /**
     * Object to delegate external entity resolving to or <code>null</code> if it's not
     * available.
     */
    private final EntityResolver entityResolver;

    /**
     * Constructs <code>XMLProcessor</code> instance, optionally using given objects to resolve
     * external entities in XML documents.
     *
     * @param entityResolver
     *     object to delegate resolving external entities to or <code>null</code> if the default
     *     handling of external entities is to be used
     */
    public XMLProcessor(EntityResolver entityResolver) {
        this.entityResolver = entityResolver;
    }

    /**
     * Processes XML document from given input stream, using given XML element handler to
     * process the root element (its sub-handlers will be used when appropriate).
     *
     * @param input stream to read XML document from
     * @param topHandler handler for the root element in the document
     * @throws SAXException in case the underlying XML parser throws an exception
     * @throws IOException in case of error reading data from the stream
     * @throws ParserConfigurationException
     *     in case the parser cannot be created with the current configuration
     */
    public void processDocument(InputStream input, ElementHandler topHandler)
        throws SAXException, IOException, ParserConfigurationException {
        SAXParser saxParser = parserFactory.newSAXParser();
        SAXHandler saxHandler = new SAXHandler(topHandler);
        saxParser.parse(input, saxHandler);
    }


    /**
     * Class responsible for handling SAX events and translating them to messages passed to the
     * appropriate {@link ElementHandler} instances.
     *
     * @author Zbigniew Chyla (zbigniew@chyla.pl)
     */
    private final class SAXHandler extends DefaultHandler {

        /**
         * Nesting level of the most innermost XML element we are currently in.
         */
        private int currentLevel;

        /**
         * Stack containing references to element handlers responsible for processing currently
         * open XML elements, together with nesting levels of these elements.
         */
        private ElementHandlersStack handlersStack = new ElementHandlersStack();

        /**
         * Flag informing whether this handler is currently accumulating characters from
         * "characters" and "ignorableWhitespace" SAX events (if not, these characters are
         * ignored).
         */
        private boolean isAccumulatingContent;

        /**
         * Used only if {@link #isAccumulatingContent} is set to <code>true</code>; contains
         * nesting level of XML element from which this handler started accumulating characters
         * (accumulating will stop after encountering the end of that element).
         */
        private int contentStartLevel;

        /**
         * Buffer used for accumulating characters from XML document if
         * {@link #isAccumulatingContent} is set to <code>true</code>.
         */
        private final StringBuffer contentBuffer = new StringBuffer();

        /**
         * Constructs <code>SAXHandler</code> instance using given handler for the top-level XML
         * element.
         *
         * @param topHandler handler to use for top-pevel XML element in the XML document
         */
        SAXHandler(ElementHandler topHandler) {
            handlersStack.push(topHandler, currentLevel);
        }

        /**
         * Handles notification of the start of an XML element.
         *
         * @param uri the name space URI
         * @param localName
         *     local name of the element or the empty string in case name space processing is
         *     not being performed
         * @param qName qualified name of the element of the empty string if it's not available
         * @param attrs given attributes of the element
         */
        public void startElement(String uri, String localName, String qName, Attributes attrs) {
            if (!isAccumulatingContent) {
                ElementHandler handler = handlersStack.peekTopHandler();
                ElementHandler subHandler = handler.getHandlerForSubElement(qName);
                if (subHandler != null) {
                    subHandler.enter(attrs);
                    handlersStack.push(subHandler, currentLevel);
                    if (!isAccumulatingContent && !subHandler.hasSubHandlers()) {
                        isAccumulatingContent = true;
                        contentStartLevel = currentLevel;
                        contentBuffer.setLength(0);
                    }
                }
            }
            currentLevel++;
        }

        /**
         * Handles notification of the end of an XML element.
         *
         * @param uri the name space URI
         * @param localName
         *     local name of the element or the empty string in case name space processing is
         *     not being performed
         * @param qName qualified name of the element of the empty string if it's not available
         */
        public void endElement(String uri, String localName, String qName) {
            currentLevel--;
            if (currentLevel == handlersStack.peekTopLevel()) {
                String elemContent;
                if (isAccumulatingContent && currentLevel == contentStartLevel) {
                    elemContent = contentBuffer.toString();
                    isAccumulatingContent = false;
                } else {
                    elemContent = "";
                }
                handlersStack.peekTopHandler().leave(elemContent);
                handlersStack.pop();
            }
        }

        /**
         * Handles notification of a sequence of characters from an XML element content.
         *
         * @param ch array containing the characters
         * @param start start index of the actual data in the array
         * @param length length of the actual data in the array
         */
        public void characters(char[] ch, int start, int length) {
            if (isAccumulatingContent) {
                contentBuffer.append(ch, start, length);
            }
        }

        /**
         * Handles notification of ignorable whitespace from an XML element content.
         *
         * @param ch array containing the characters
         * @param start start index of the actual data in the array
         * @param length length of the actual data in the array
         */
        public void ignorableWhitespace(char[] ch, int start, int length) {
            if (isAccumulatingContent) {
                contentBuffer.append(ch, start, length);
            }
        }

        /**
         * Resolves an external entity by delegating it to a private resolver object.
         * <p>
         * Note:<br>
         * Because of incompatibility between JDK 1.5/5.0 and JDK 1.4 this method must wrap
         * <code>IOException</code> with <code>SAXException</code>. Thanks to this approach it
         * compiles and works fine on both versions.
         *
         * @param publicId the public identifier
         * @param systemId the system identifier or <code>null</code> if not available
         * @return
         *     input source providing external entity or <code>null</code> if the default access
         *     method should be used
         * @throws SAXException
         *     in case a method actually resolving the entity throws an exception
         */
        public InputSource resolveEntity(String publicId, String systemId) throws SAXException {
            InputSource src;
            try {
                if (entityResolver != null) {
                    src = entityResolver.resolveEntity(publicId, systemId);
                    if (src == null) {
                        src = super.resolveEntity(publicId, systemId);
                    }
                } else {
                    src = super.resolveEntity(publicId, systemId);
                }
            } catch (IOException e) {
                throw new SAXException(e);
            }
            return src;
        }

    }

}
